"""
Kross Script Editor

Description:
This script provides a in python written script editor.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

import os, sys

try:
	import qt
except (ImportError):
	raise "Failed to import the required PyQt python module."

####################################################################################
# Samples.

class Widget(qt.QHBox):
	def __init__(self, parentwidget, label = None):
		self.parentwidget = parentwidget
		import qt
		qt.QHBox.__init__(self, parentwidget)
		self.setMargin(4)
		self.setSpacing(4)
		if label != None: qt.QLabel(label, self)
	def value(self):
		return None

class ListWidget(Widget):
	def __init__(self, parentwidget, label):
		import qt
		global Widget
		Widget.__init__(self, parentwidget, label)
		self.combo = qt.QComboBox(self)
		self.combo.setEditable(True)
		self.setStretchFactor(self.combo,1)
	def value(self):
		return self.combo.currentText()

class EditWidget(Widget):
	def __init__(self, parentwidget, label):
		import qt
		global Widget
		Widget.__init__(self, parentwidget, label)
		self.edit = qt.QLineEdit(self)
		self.setStretchFactor(self.edit,1)
	def value(self):
		return self.edit.text()

class FileWidget(Widget):
	def __init__(self, parentwidget, label, filtermask, openfiledialog = True):
		self.filtermask = filtermask
		self.openfiledialog = openfiledialog
		import qt
		global Widget
		Widget.__init__(self, parentwidget, label)
		self.edit = qt.QLineEdit(self)
		self.setStretchFactor(self.edit,1)
		btn = qt.QPushButton("...",self)
		qt.QObject.connect(btn, qt.SIGNAL("clicked()"), self.btnClicked)
	def btnClicked(self):
		import qt
		text = str( self.edit.text() )
		if self.openfiledialog:
			filename = str( qt.QFileDialog.getOpenFileName(text, self.filtermask, self.parentwidget) )
		else:
			filename = qt.QFileDialog.getSaveFileName(text, self.filtermask, self.parentwidget)
		if filename != "": self.edit.setText( filename )
	def value(self):
		return self.edit.text()

class Samples:

	####################################################################################
	# KexiDB

	class KexiDB:
		def __init__(self, parentwidget):
			self.parentwidget = parentwidget

		class _ProjectWidget(FileWidget):
			def __init__(self, parentwidget):
				global FileWidget
				FileWidget.__init__(self, parentwidget, "Project File:", "*.kexi *.kexis *.kexic;;*")

		class _DriverWidget(ListWidget):
			def __init__(self, parentwidget):
				global ListWidget
				ListWidget.__init__(self, parentwidget, "Driver:")
				import krosskexidb
				for driver in krosskexidb.DriverManager().driverNames():
					self.combo.insertItem(driver)

		class _TableWidget(ListWidget):
			def __init__(self, parentwidget):
				global ListWidget
				ListWidget.__init__(self, parentwidget, "Table:")

		class PrintDriverDetails:
			"""  Print a the list of available KexiDB drivers and print details about one of them. """
			name = "Details about a driver"
			def __init__(self, parent):
				self.widgets = {
					"DriverName" : Samples.KexiDB._DriverWidget( parent.parentwidget ),
				}
			def getCode(self):
				return (
					'import krosskexidb',
					'drivermanager = krosskexidb.DriverManager()',
					'print "drivernames: %s" % drivermanager.driverNames()',
					'',
					'driver = drivermanager.driver( \"{DriverName}\" )',
					'print "driver: {DriverName}"',
					'print "version=%s.%s" % (driver.versionMajor(),driver.versionMinor())',
					'print "mimetype=%s" % driver.fileDBDriverMimeType()',
					'print "filedriver=%s" % driver.isFileDriver()',
				)

		class ConnectWithFile:
			""" Connect with a KexiDB database by using a Kexi Connection Project File. """
			name = "Connect with file"
			def __init__(self, parent):
				self.widgets = {
					"ProjectFile" : Samples.KexiDB._ProjectWidget( parent.parentwidget ),
				}
			def getCode(self):
				return (
					'# Import the KexiDB module.',
					'import krosskexidb',
					'drivermanager = krosskexidb.DriverManager()',
					'',
					'# Get the connectiondata from the project file.',
					'connectiondata = drivermanager.createConnectionDataByFile( "{ProjectFile}" )',
					'print "Connectiondata: %s" % connectiondata.serverInfoString()',
					'',
					'# Create the driver for the database backend.',
					'driver = drivermanager.driver( connectiondata.driverName() )',
					'',
					'# Create and establish the connection to the database.',
					'connection = driver.createConnection(connectiondata)',
					'if not connection.isConnected():',
					'	if not connection.connect():',
					'		raise "Failed to connect"',
					'',
					'# Open the database for usage.',
					'print "Databases: %s" % connection.databaseNames()',
					'if not connection.isDatabaseUsed():',
					'	if not connection.useDatabase( connectiondata.databaseName() ):',
					'		if not connection.useDatabase( connectiondata.fileName() ):',
					'			raise "Failed to use database"',
					'',
					'# Print some infos.',
					'print "All tables: %s" % connection.allTableNames()',
					'print "Tables: %s" % connection.tableNames()',
					'print "Queries: %s" % connection.queryNames()',
				)

		class IterateThroughTable:
			""" Iterate through a table within a connected KexiDB database. """
			name = "Iterate through table"
			def __init__(self, parent):
				self.widgets = {
					"ProjectFile" : Samples.KexiDB._ProjectWidget( parent.parentwidget ),
					"TableName" : Samples.KexiDB._TableWidget( parent.parentwidget ),
				}
			def getCode(self):
				return (
					'# Import the KexiDB module.',
					'import krosskexidb',
					'drivermanager = krosskexidb.DriverManager()',
					'',
					'# Get the connectiondata from the project file.',
					'connectiondata = drivermanager.createConnectionDataByFile( "{ProjectFile}" )',
					'print "Connectiondata: %s" % connectiondata.serverInfoString()',
					'',
					'# Create the driver for the database backend.',
					'driver = drivermanager.driver( connectiondata.driverName() )',
					'',
					'# Create and establish the connection to the database.',
					'connection = driver.createConnection(connectiondata)',
					'if not connection.isConnected():',
					'	if not connection.connect():',
					'		raise "Failed to connect"',
					'',
					'# Open the database for usage.',
					'if not connection.isDatabaseUsed():',
					'	if not connection.useDatabase( connectiondata.databaseName() ):',
					'		if not connection.useDatabase( connectiondata.fileName() ):',
					'			raise "Failed to use database"',
					'',
					'# Get the table and create a query for it.',
					'table = connection.tableSchema( \"{TableName}\" )',
					'query = table.query()',
					'',
					'# Create a cursor to walk through the records.',
					'cursor = connection.executeQuerySchema( query )',
					'if not cursor:',
					'	raise "Failed to create cursor."',
					'',
					'# Iterate through the records.',
					'if not cursor.moveFirst():',
					'	raise "The cursor has no records to read from."',
					'while not cursor.eof():',
					'	for i in range( cursor.fieldCount() ):',
					'		print "%s" % cursor.value(i)',
					'	cursor.moveNext()',
				)

	####################################################################################
	# KSpread

	class KSpread:
		def __init__(self, parentwidget):
			self.parentwidget = parentwidget

		class _SheetWidget(ListWidget):
			def __init__(self, parentwidget, label = "Sheet:"):
				global ListWidget
				ListWidget.__init__(self, parentwidget, label)

				try:
					import krosskspreadcore
					document = krosskspreadcore.get("KSpreadDocument")
					for sheetname in document.sheetNames():
						self.combo.insertItem(sheetname)
				except:
					import traceback
					trace = "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
					print trace

		class _CellsWidget(ListWidget):
			def __init__(self, parentwidget):
				global ListWidget
				ListWidget.__init__(self, parentwidget, "Cells (col1:row1 - col2:row2):")
				self.combo.insertItem( "1:1 - %s:%s" % (5,10) )
				self.combo.insertItem( "1:1 - %s:%s" % (256,256) )
				self.combo.insertItem( "1:1 - %s:%s" % (32767,32767) )
			def value(self):
				return [ [ int(i) for i in item.split(':') ] for item in str( ListWidget.value(self) ).split('-') ]

		class _ValueWidget(EditWidget):
			def __init__(self, parentwidget):
				global EditWidget
				EditWidget.__init__(self, parentwidget, "Value:")
				self.edit.setText("Some text")

		class _ColorWidget(EditWidget):
			def __init__(self, parentwidget,label,color):
				global EditWidget
				EditWidget.__init__(self, parentwidget, "%s (RGB):" % label)
				self.edit.setText(color)
			def value(self):
				return "#%s" % EditWidget.value(self)

		class SetTextOfCells:
			""" Set the text of the defined cells. """
			name = "Set text of cells"
			def __init__(self, parent):
				pass
				self.widgets = {
					"SheetName" : Samples.KSpread._SheetWidget( parent.parentwidget ),
					"Cells" : Samples.KSpread._CellsWidget( parent.parentwidget ),
					"Value" : Samples.KSpread._ValueWidget( parent.parentwidget ),
				}
			def getCode(self):
				return (
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'',
					'# Get the current document.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'',
					'# Get the sheet defined by the sheetname.',
					'sheet = document.sheetByName( \"{SheetName}\" )',
					'if not sheet:',
					'	raise "No such sheet {SheetName} %s" % document.sheetNames()',
					'',
					'( (col1,row1),(col2,row2) ) = {Cells}',
					'for c in range(col1,col2):',
					'	for r in range(row1,row2):',
					'		cell = sheet.cell(c,r)',
					'		print "cell c=%s r=%s v=%s" % (c,r,cell.value())',
					'		cell.setText( \"{Value}\" )',
				)

		class SetColorsOfCells:
			""" Set the colors of the defined cells. """
			name = "Set colors of cells"
			def __init__(self, parent):
				pass
				self.widgets = {
					"SheetName" : Samples.KSpread._SheetWidget( parent.parentwidget),
					"Cells" : Samples.KSpread._CellsWidget( parent.parentwidget ),
					"TextColor" : Samples.KSpread._ColorWidget( parent.parentwidget, "Textcolor", "ff0000" ),
					"BackgroundColor" : Samples.KSpread._ColorWidget( parent.parentwidget, "Backgroundcolor", "c0c0c0" ),
				}
			def getCode(self):
				return (
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'',
					'# Get the current document.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'',
					'# Get the sheet defined by the sheetname.',
					'sheet = document.sheetByName( \"{SheetName}\" )',
					'if not sheet:',
					'	raise "No such sheet {SheetName} %s" % document.sheetNames()',
					'',
					'( (col1,row1),(col2,row2) ) = {Cells}',
					'for c in range(col1,col2):',
					'	for r in range(col1,col2):',
					'		cell = sheet.cell(c,r)',
					'		cell.setTextColor( \"{TextColor}\" )',
					'		cell.setBackgroundColor( \"{BackgroundColor}\" )',
				)

		class IterateThroughCellsWithContent:
			""" Iterate over all cells in a sheet that have content (aka that are not empty). """
			name = "Iterate through cells"
			def __init__(self, parent):
				pass
				self.widgets = {
					"SheetName" : Samples.KSpread._SheetWidget( parent.parentwidget ),
				}
			def getCode(self):
				return (
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'',
					'# Get the current document.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'',
					'# Get the sheet defined by the sheetname.',
					'sheet = document.sheetByName( \"{SheetName}\" )',
					'if not sheet:',
					'	raise "No such sheet {SheetName} %s" % document.sheetNames()',
					'',
					'# Iterate through the cells that have content (aka that are not empty).',
					'cell = sheet.firstCell()',
					'while cell:',
					'	print "col=%s row=%s value=%s" % (cell.column(),cell.row(),cell.value())',
					'	cell = cell.nextCell()',
				)

		class PrintSheetDetails:
			""" Print details about the current sheet. """
			name = "Details about a sheet"
			def __init__(self, parent):
				self.widgets = {
					"SheetName" : Samples.KSpread._SheetWidget( parent.parentwidget ),
				}
			def getCode(self):
				return (
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'',
					'# Get the current document.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'',
					'# Get the sheet defined by the sheetname.',
					'sheet = document.sheetByName( \"{SheetName}\" )',
					'if not sheet:',
					'	raise "No such sheet {SheetName} %s" % document.sheetNames()',
					'',
					'print "name=%s" % sheet.name()',
					'print "maxcolumns=%s maxrows=%s" % (sheet.maxColumn(),sheet.maxRow())',
				)

		class LoadDocFromNativeXML:
			""" Load the document from a native XML file. """
			name = "Load document from native XML File"
			def __init__(self, parent):
				global FileWidget
				self.widgets = {
					"FileName" : FileWidget( parent.parentwidget, "XML File:", "*.xml;;*" ),
				}
			def getCode(self):
				return (
					'# Import the PyQt module.',
					'import qt',
					'',
					'def loadFile(filename):',
					'	# Import the krosskspreadcore module.',
					'	import krosskspreadcore',
					'	# Try to read the file.',
					'	try:',
					'		file = open(filename, "r")',
					'		xml = file.read()',
					'		file.close()',
					'	except IOError, (errno, strerror):',
					'		qt.QMessageBox.critical(self,"Error","<qt>Failed to read file %s<br><br>%s</qt>" % (filename,strerror))',
					'		return',
					'',
					'	# Get the current document.',
					'	document = krosskspreadcore.get("KSpreadDocument")',
					'	# Load the document from the native XML string.',
					'	ok = document.loadNativeXML( xml )',
					'',
					'# Show the openfile dialog',
					'filename = "{FileName}"',
					'openfilename = qt.QFileDialog.getOpenFileName(filename,"*.xml;;*", self)',
					'if str(openfilename) != "":',
					'	loadFile( openfilename )',
				)

		class SaveDocToNativeXML:
			""" Save the document to a native XML file. """
			name = "Save document to native XML File"
			def __init__(self, parent):
				global FileWidget
				self.widgets = {
					"FileName" : FileWidget( parent.parentwidget, "XML File:", "*.xml;;*", False ),
				}
			def getCode(self):
				return (
					'# Import the PyQt module.',
					'import qt',
					'',
					'def saveFile(filename):',
					'	# Import the krosskspreadcore module.',
					'	import krosskspreadcore',
					'	# Try to open the file for writting.',
					'	try:',
					'		file = open(filename, "w")',
					'	except IOError, (errno, strerror):',
					'		qt.QMessageBox.critical(self,"Error","<qt>Failed to create file %s<br><br>%s</qt>" % (filename,strerror))',
					'		return',
					'	# Get the current document.',
					'	document = krosskspreadcore.get("KSpreadDocument")',
					'	# Get the native XML string.',
					'	xml = document.saveNativeXML()',
					'	# Write the XML string to the file.',
					'	file.write( xml )',
					'	# Close the file.',
					'	file.close()',
					'',
					'# Show the savefile dialog',
					'filename = "{FileName}"',
					'savefilename = qt.QFileDialog.getSaveFileName(filename,"*.xml;;*", self)',
					'if str(savefilename) != "":',
					'	saveFile( savefilename )',
				)

		class CopySheets:
			""" Copy the text-content from one sheet to another. """
			name = "Copy sheets"
			def __init__(self, parent):
				self.widgets = {
					"SourceSheet" : Samples.KSpread._SheetWidget( parent.parentwidget, "Source sheet:" ),
					"TargetSheet" : Samples.KSpread._SheetWidget( parent.parentwidget, "Target sheet:" ),
				}
			def getCode(self):
				return (
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'# Get the current document.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'# Get the source sheet.',
					'fromsheet = document.sheetByName( "{SourceSheet}" )',
					'if not fromsheet: raise "No such sheet {SourceSheet} %s" % document.sheetNames()',
					'# Get the target sheet.',
					'tosheet = document.sheetByName( "{TargetSheet}" )',
					'if not fromsheet: raise "No such sheet {TargetSheet} %s" % document.sheetNames()',
					'# Copy the cells.',
					'fromcell = fromsheet.firstCell()',
					'while fromcell:',
					'	tocell = tosheet.cell( fromcell.column(), fromcell.row() )',
					'	tocell.setText( fromcell.text() )',
					'	#tocell.setValue( fromcell.value() )',
					'	fromcell = fromcell.nextCell()',
				)

		class LoadSheetFromCSV:
			""" Load the content of a CSV file into a KSpread sheet. """
			name = "Load data from CSV file into sheet"
			def __init__(self, parent):
				self.widgets = {
					"Sheet" : Samples.KSpread._SheetWidget( parent.parentwidget ),
					"FileName" : FileWidget( parent.parentwidget, "CSV File:", "*.csv;;*", True ),
				}
			def getCode(self):
				return (
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'# Get the current document and the sheet.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'sheet = document.sheetByName( "{Sheet}" )',
					'if not sheet: raise "No such sheet {Sheet} %s" % document.sheetNames()',
					'',
					'filename = "{FileName}"',
					'try:',
					'	file = open(filename, "r")',
					'except IOError:',
					'	raise "Failed to open CSV File: %s" % filename',
					'',
					'import csv',
					'csvparser = csv.reader(file)',
					'row = 1',
					'while True:',
					'	try:',
					'		record = csvparser.next()',
					'	except StopIteration:',
					'		break',
					'	col = 1',
					'	for item in record:',
					'		sheet.cell(col,row).setText( item )',
					'		col += 1',
					'	row += 1',
					'file.close()',
				)

		class SaveSheetToCSV:
			""" Save the content of a KSpread sheet into a CSV file. """
			name = "Save data from a sheet into a CSV file"
			def __init__(self, parent):
				self.widgets = {
					"Sheet" : Samples.KSpread._SheetWidget( parent.parentwidget ),
					"FileName" : FileWidget( parent.parentwidget, "CSV File:", "*.csv;;*", False ),
				}
			def getCode(self):
				return (
					'filename = "{FileName}"',
					'try:',
					'	file = open(filename, "w")',
					'except IOError:',
					'	raise "Failed to write CSV File: %s" % filename',
					'# Prepare CSV-writer',
					'import csv',
					'csvwriter = csv.writer(file)',
					'# Import the KSpread module.',
					'import krosskspreadcore',
					'# Get the current document and the sheet.',
					'document = krosskspreadcore.get("KSpreadDocument")',
					'sheet = document.sheetByName( "{Sheet}" )',
					'if not sheet: raise "No such sheet {Sheet} %s" % document.sheetNames()',
					'# Iterate over the cells.',
					'cell = sheet.firstCell()',
					'record = []',
					'while cell:',
					'	record.append( cell.text() )',
					'	if cell.column() == 1 or not cell.nextCell():',
					'		csvwriter.writerow( record )',
					'		record = []',
					'	cell = cell.nextCell()',
					'file.close()',
				)


	####################################################################################
	# PyQt

	class PyQt:
		def __init__(self, parentwidget):
			self.parentwidget = parentwidget

		class OpenFileDialog:
			""" Show the usage of the openfile dialog with QFileDialog. """
			name = "Open File Dialog"
			def __init__(self, parent):
				pass
				self.widgets = {
					"FileName" : FileWidget( parent.parentwidget, "Open File:", "*.txt *.html;;*" ),
				}
			def getCode(self):
				return (
					'import qt',
					'openfilename = qt.QFileDialog.getOpenFileName("{FileName}","*.txt *.html;;*", self)',
					'print "openfile=%s" % openfilename',
				)

		class SaveFileDialog:
			""" Show the usage of the savefile dialog with QFileDialog. """
			name = "Save File Dialog"
			def __init__(self, parent):
				pass
				self.widgets = {
					"FileName" : FileWidget( parent.parentwidget, "Save File:", "*.txt *.html;;*", False ),
				}
			def getCode(self):
				return (
					'import qt',
					'savefilename = qt.QFileDialog.getSaveFileName("{FileName}","*.txt *.html;;*", self)',
					'print "savefile=%s" % savefilename',
				)

		class CustomDialog:
			""" Show a custom dialog that inherits a QDialog. """
			name = "Custom Dialog"
			def __init__(self, parent):
				pass
				self.widgets = {
				}
			def getCode(self):
				return (
					'import qt',
					'',
					'class MyDialog(qt.QDialog):',
					'	def __init__(self, parent):',
					'		import qt',
					'		qt.QDialog.__init__(self, parent, "MyDialog", 1, qt.Qt.WDestructiveClose)',
					'		self.setCaption("My Dialog")',
					'		btn = qt.QPushButton("Click me",self)',
					'		qt.QObject.connect(btn, qt.SIGNAL("clicked()"), self.buttonClicked)',
					'	def buttonClicked(self):',
					'		import qt',
					'		qt.QMessageBox.information(self, "The Caption", "This is the message string.")',
					'',
					'dialog = MyDialog(self)',
					'dialog.exec_loop()',
				)

		class InputDialog:
			""" Show how to use a QInputDialog. """
			name = "Input Dialog"
			def __init__(self, parent):
				global EditWidget
				self.widgets = {
					"Caption" : EditWidget( parent.parentwidget, "Caption" ),
					"Message" : EditWidget( parent.parentwidget, "Message" ),
				}
			def getCode(self):
				return (
					'import qt',
					'',
					'text, ok = qt.QInputDialog.getText("{Caption}", "{Message}", qt.QLineEdit.Normal, "")',
					'if ok:',
					'	print "Text defined: %s" % text',
					'else:',
					'	print "Dialog aborted."',
				)

	####################################################################################
	# DCOP

	class DCOP:
		def __init__(self, parentwidget):
			self.parentwidget = parentwidget

		class PrintClipboard:
			""" Print the content from the clipper via DCOP. """
			name = "Clipboard content"
			def __init__(self, parent):
				self.widgets = {
				}
			def getCode(self):
				return (
					'import qt, kdecore, dcop, dcopext',
					'dcopclient = kdecore.KApplication.dcopClient()',
					'apps = [ app for app in dcopclient.registeredApplications() if str(app).startswith("klipper") ]',
					'd = dcopext.DCOPApp(apps[0], dcopclient)',
					'result,typename,data = d.appclient.call(apps[0],"klipper","getClipboardContents()","")',
					'ds = qt.QDataStream(data, qt.IO_ReadOnly)',
					'print "Clipboard content:\\n%s" % kdecore.dcop_next(ds, "QString")',
				)

		class AmarokCollectionInfos:
			""" Fetch some collection informations from the amarok collection via DCOP. """
			name = "amarok collection infos"
			def __init__(self, parent):
				self.widgets = {
				}
			def getCode(self):
				return (
					'import qt, kdecore, dcop, dcopext',
					'',
					'dcopclient = kdecore.KApplication.dcopClient()',
					'apps = [ app for app in dcopclient.registeredApplications() if str(app).startswith("amarok") ]',
					'app = apps[0]',
					'd = dcopext.DCOPApp(app, dcopclient)',
					'',
					'def dataToList(data, types = []):',
					'	import qt, kdecore',
					'	ds = qt.QDataStream(data, qt.IO_ReadOnly)',
					'	return [ kdecore.dcop_next(ds, t) for t in types ]',
					'',
					'for funcname in ["totalAlbums","totalArtists","totalCompilations","totalGenres","totalTracks"]:',
					'	result,replytype,replydata = d.appclient.call("amarok", "collection", "%s()" % funcname,"")',
					'	print "%s: %s" % ( funcname, dataToList(replydata,["int"])[0] )',
				)

		class KopeteContacts:
			""" Print the names of all contacts Kopete knows via DCOP. """
			name = "Kopete contacts"
			def __init__(self, parent):
				self.widgets = {
				}
			def getCode(self):
				return (
					'import qt, kdecore, dcop, dcopext',
					'',
					'dcopclient = kdecore.KApplication.dcopClient()',
					'apps = [ app for app in dcopclient.registeredApplications() if str(app).startswith("kopete") ]',
					'app = apps[0]',
					'd = dcopext.DCOPApp(app, dcopclient)',
					'',
					'(state,rtype,rdata) = d.appclient.call("kopete", "KopeteIface", "contacts()","")',
					'if not state: raise "Failed to call the kopete contacts-function"',
					'',
					'ds = qt.QDataStream(rdata.data(), qt.IO_ReadOnly)',
					'sl = kdecore.dcop_next (ds, "QStringList")',
					'print "contacts=%s" % [ str(s) for s in sl ]',
				)

		class KWordSelectedText:
			""" Get the selected text from a KWord instance via DCOP. """
			name = "KWord selected text"
			def __init__(self, parent):
				self.widgets = {
				}
			def getCode(self):
				return (
					'import qt, kdecore, dcop, dcopext',
					'',
					'def dataToList(data, types = []):',
					'	import qt, kdecore',
					'	ds = qt.QDataStream(data, qt.IO_ReadOnly)',
					'	return [ kdecore.dcop_next(ds, t) for t in types ]',
					'def listToData(listdict):',
					'	import qt, kdecore',
					'	ba= qt.QByteArray()',
					'	ds = qt.QDataStream(ba, qt.IO_WriteOnly)',
					'	for (typename,value) in listdict:',
					'		kdecore.dcop_add (ds, value, typename)',
					'	return ba',
					'',
					'# Get the KWord DCOP client.',
					'dcopclient = kdecore.KApplication.dcopClient()',
					'apps = [ app for app in dcopclient.registeredApplications() if str(app).startswith("kword") ]',
					'if len(apps) < 1: raise "No KWord instance is running. Please start KWord before!"',
					'appname = apps[0]',
					'd = dcopext.DCOPApp(appname, dcopclient)',
					'',
					'# Call the getDocuments() function.',
					'(state,rtype,rdata) = d.appclient.call(appname, "KoApplicationIface", "getDocuments()","")',
					'if not state: raise "%s: Failed to call getDocuments-function" % appname',
					'documents = dataToList(rdata,["QValueList<DCOPRef>"])[0]',
					'print "documents=%s" % [ str( doc.obj() ) for doc in documents ]',
					'document = documents[0] # Let\'s just take the first document.',
					'',
					'# Get the frameset.',
					'ba = listToData( [ ("int",0) ] )',
					'(state,rtype,rdata) = d.appclient.call(appname, document.obj(), "textFrameSet(int)", ba)',
					'if not state: raise "%s: Failed to call frameSet-function" % appname',
					'frameset = dataToList( rdata,["DCOPRef"] )[0] # Let\'s just take the first textframe.',
					'',
					'# Get the selected text.',
					'(state,rtype,rdata) = d.appclient.call(appname, frameset.obj(), "selectedText()", "")',
					'print "Selected Text: %s" % dataToList( rdata,["QString"] )[0]',
				)

####################################################################################
# Dialog implementations.

class SampleDialog(qt.QDialog):
	def __init__(self, parent, sampleclazz, samplechildclazz):
		import qt
		qt.QDialog.__init__(self, parent, "SampleDialog", 1)

		layout = qt.QVBoxLayout(self)
		box = qt.QVBox(self)
		box.setMargin(4)
		box.setSpacing(10)
		layout.addWidget(box)

		self.scrollview = qt.QScrollView(box)
		self.scrollview.setResizePolicy(qt.QScrollView.AutoOne)
		#self.scrollview.setFrameStyle(qt.QFrame.NoFrame);
		self.scrollview.setResizePolicy(qt.QScrollView.AutoOneFit);
		self.scrollview.viewport().setPaletteBackgroundColor(self.paletteBackgroundColor())
		mainbox = qt.QVBox( self.scrollview.viewport() )
		mainbox.setMargin(6)
		mainbox.setSpacing(6)
		desclabel = qt.QLabel(mainbox)
		qt.QFrame(mainbox).setFrameStyle( qt.QFrame.HLine | qt.QFrame.Sunken )

		self.sample = sampleclazz( mainbox )
		self.samplechild = samplechildclazz( self.sample )

		desclabel.setText( "<qt>%s</qt>" % self.samplechild.__doc__ )
		mainbox.setStretchFactor(qt.QWidget(mainbox), 1)
		mainbox.show()
		self.scrollview.addChild(mainbox)

		btnbox = qt.QHBox(box)
		btnbox.setMargin(6)
		btnbox.setSpacing(6)
		okbtn = qt.QPushButton(btnbox)
		okbtn.setText("Ok")
		qt.QObject.connect(okbtn, qt.SIGNAL("clicked()"), self.okClicked)
		cancelbtn = qt.QPushButton(btnbox)
		cancelbtn.setText("Cancel")
		qt.QObject.connect(cancelbtn, qt.SIGNAL("clicked()"), self.close)

		self.setCaption(self.samplechild.name)
		box.setMinimumSize(qt.QSize(500,340))

	def okClicked(self):
		self.code = self.samplechild.getCode()
		self.close()

	def getCode(self):
		if not hasattr(self,"code"): return None
		code = "\n".join( self.code )
		for widgetname in self.samplechild.widgets.keys():
			print ".............. %s" % widgetname
			widget = self.samplechild.widgets[widgetname]
			value = widget.value()
			if value != None:
				code = code.replace("{%s}" % widgetname, str(value))
		return code

class MainDialog(qt.QDialog):
	def __init__(self, scriptpath, parent):
		self.scriptpath = scriptpath
		if not hasattr(__main__,"scripteditorfilename"):
			__main__.scripteditorfilename = self.getFileName("myscript.py")

		import krosskspreadcore
		self.doc = krosskspreadcore.get("KSpreadDocument")

		import os, qt
		qt.QDialog.__init__(self, parent, "MainDialog", 1, qt.Qt.WDestructiveClose)
		self.setCaption("Script Editor")
		
		layout = qt.QVBoxLayout(self)
		box = qt.QVBox(self)
		box.setMargin(4)
		box.setSpacing(10)
		layout.addWidget(box)

		menu = qt.QMenuBar(box)

		splitter = qt.QSplitter(box)
		splitter.setOrientation(qt.Qt.Vertical)

		self.scripttext = qt.QMultiLineEdit(splitter)
		self.scripttext.setWordWrap( qt.QTextEdit.NoWrap )
		self.scripttext.setTextFormat( qt.Qt.PlainText )
		qt.QObject.connect(self.scripttext, qt.SIGNAL("cursorPositionChanged(int,int)"),self.cursorPositionChanged)

		self.console = qt.QTextBrowser(splitter)
		splitter.setResizeMode(self.console, qt.QSplitter.KeepSize)

		statusbar = qt.QStatusBar(box)
		self.messagestatus = qt.QLabel(statusbar)
		statusbar.addWidget(self.messagestatus,1)
		self.cursorstatus = qt.QLabel(statusbar)
		statusbar.addWidget(self.cursorstatus)
		self.cursorPositionChanged()

		box.setMinimumSize( qt.QSize(680,540) )

		filemenu = qt.QPopupMenu(menu)
		menu.insertItem("&File", filemenu)
		
		newaction = qt.QAction("New", qt.QKeySequence("CTRL+N"), self)
		qt.QObject.connect(newaction, qt.SIGNAL("activated()"), self.newFile)
		newaction.addTo(filemenu)

		openaction = qt.QAction("Open...", qt.QKeySequence("CTRL+O"), self)
		qt.QObject.connect(openaction, qt.SIGNAL("activated()"), self.openFileAs)
		openaction.addTo(filemenu)

		saveaction = qt.QAction("Save", qt.QKeySequence("CTRL+S"), self)
		qt.QObject.connect(saveaction, qt.SIGNAL("activated()"), self.saveFile)
		saveaction.addTo(filemenu)

		saveasaction = qt.QAction("Save as...", qt.QKeySequence("CTRL+A"), self)
		qt.QObject.connect(saveasaction, qt.SIGNAL("activated()"), self.saveFileAs)
		saveasaction.addTo(filemenu)

		filemenu.insertSeparator()
		
		quitaction = qt.QAction("Quit", qt.QKeySequence("CTRL+Q"), self)
		qt.QObject.connect(quitaction, qt.SIGNAL("activated()"), self.close)
		quitaction.addTo(filemenu)

		editmenu = qt.QPopupMenu(menu)
		menu.insertItem("&Edit", editmenu)

		undoaction = qt.QAction("Undo", qt.QKeySequence("CTRL+Z"), self)
		qt.QObject.connect(undoaction, qt.SIGNAL("activated()"), self.scripttext.undo)
		undoaction.addTo(editmenu)

		redoaction = qt.QAction("Redo", qt.QKeySequence("CTRL+Shift+Z"), self)
		qt.QObject.connect(redoaction, qt.SIGNAL("activated()"), self.scripttext.redo)
		redoaction.addTo(editmenu)

		editmenu.insertSeparator()

		cutaction = qt.QAction("Cut", qt.QKeySequence("CTRL+X"), self)
		qt.QObject.connect(cutaction, qt.SIGNAL("activated()"), self.scripttext.cut)
		cutaction.addTo(editmenu)

		copyaction = qt.QAction("Copy", qt.QKeySequence("CTRL+C"), self)
		qt.QObject.connect(copyaction, qt.SIGNAL("activated()"), self.scripttext.copy)
		copyaction.addTo(editmenu)

		pasteaction = qt.QAction("Paste", qt.QKeySequence("CTRL+V"), self)
		qt.QObject.connect(pasteaction, qt.SIGNAL("activated()"), self.scripttext.paste)
		pasteaction.addTo(editmenu)

		clearaction = qt.QAction("Clear", qt.QKeySequence("CTRL+Shift+X"), self)
		qt.QObject.connect(clearaction, qt.SIGNAL("activated()"), self.scripttext.clear)
		clearaction.addTo(editmenu)

		editmenu.insertSeparator()

		selallaction = qt.QAction("Select All", 0, self)
		qt.QObject.connect(selallaction, qt.SIGNAL("activated()"), self.scripttext.selectAll)
		selallaction.addTo(editmenu)

		scriptmenu = qt.QPopupMenu(menu)
		menu.insertItem("&Script", scriptmenu)

		compileaction = qt.QAction("Compile", qt.QKeySequence("F9"), self)
		qt.QObject.connect(compileaction, qt.SIGNAL("activated()"), self.compileScript)
		compileaction.addTo(scriptmenu)

		executeaction = qt.QAction("Execute", qt.QKeySequence("F10"), self)
		qt.QObject.connect(executeaction, qt.SIGNAL("activated()"), self.executeScript)
		executeaction.addTo(scriptmenu)

		self.samplemenu = qt.QPopupMenu(menu)
		menu.insertItem("&Samples", self.samplemenu)
		itemid = 500
		global Samples
		for samplename in dir(Samples):
			if samplename.startswith("_"): continue
			itemid += 1
			menu = qt.QPopupMenu(self.samplemenu)
			qt.QObject.connect(menu, qt.SIGNAL("activated(int)"), self.sampleActivated)
			self.samplemenu.insertItem(samplename, menu, -1, self.samplemenu.count() - 1)
			attr = getattr(Samples,samplename)
			for a in dir(attr):
				if a.startswith("_"): continue
				itemid += 1
				child = getattr(attr,a)
				itemid = menu.insertItem(child.name, itemid)
				menu.setWhatsThis(itemid,"%s/%s" % (samplename,a))

		if os.path.exists(__main__.scripteditorfilename):
			self.openFile(__main__.scripteditorfilename)

	def getFileName(self, filename):
		import os
		try:
			homepath = os.getenv("HOME")
			if not homepath:
				import pwd
				user = os.getenv("USER") or os.getenv("LOGNAME")
				if not user:
					pwent = pwd.getpwuid(os.getuid())
				else:
					pwent = pwd.getpwnam(user)
				homepath = pwent[6]
		except (KeyError, ImportError):
			homepath = os.curdir
		return os.path.join(homepath, filename)

	def cursorPositionChanged(self,para = 0,pos = 0):
		self.cursorstatus.setText( "Line: %s Col: %s" % (para+1,pos+1) )

	def sampleActivated(self, index):
		global Samples
		sampleid = str( self.sender().whatsThis(index) )
		sampleidlist = sampleid.split('/')
		sampleclazz = getattr( Samples,sampleidlist[0] )
		samplechildclazz = getattr( sampleclazz, sampleidlist[1] )

		global SampleDialog
		dialog = SampleDialog(self, sampleclazz, samplechildclazz)
		dialog.exec_loop()
		code = dialog.getCode()
		if code != None:
			self.scripttext.append( code )

	def execCode(self,function):
		import sys, StringIO
		codeOut = StringIO.StringIO()
		codeErr = StringIO.StringIO()
		sys.stdout = codeOut
		sys.stderr = codeErr

		try:
			function(self)
		except:
			import traceback
			trace = "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			sys.stderr.write(trace)
			try:
				# this is a bit tricky. we need to go to steps back to know where the exception really happened.
				tb = sys.exc_info()[2]
				while hasattr(tb,"tb_next") and tb.tb_next:
					tb = tb.tb_next
				lineno = tb.tb_lineno
				print "EXCEPTION: lineno=%s" % lineno
				self.scripttext.setCursorPosition( lineno - 1, 0 )
			except:
				pass

		sys.stdout = sys.__stdout__
		sys.stderr = sys.__stderr__

		s = codeErr.getvalue()
		if s:
			print "ERROR:\n%s\n" % s
			self.console.append(s)

		s = codeOut.getvalue()
		if s:
			print s
			self.console.append(s)

		codeOut.close()
		codeErr.close()

	def compileScript(self):
		self.console.clear()
		code = str( self.scripttext.text() )
		def docompile(self):
			compile(code, __main__.scripteditorfilename, 'exec')
		self.execCode(docompile)
		self.console.append("<b>Compiled!</b>")

	def executeScript(self):
		self.console.clear()
		def doexecute(self):
			code = str( self.scripttext.text() )
			exec code in globals(), locals()
		self.execCode(doexecute)
		self.console.append("<b>Execution done!</b>")

	def newFile(self):
		self.console.clear()
		#if qt.QMessageBox.warning(self,"Remove?","Remove the selected item?",qt.QMessageBox.Yes,qt.QMessageBox.Cancel) != qt.QMessageBox.Yes:
		self.scripttext.clear()

	def openFile(self, filename):
		__main__.scripteditorfilename = None
		try:
			file = open(filename, "r")
			self.scripttext.setText( str( file.read() ) )
			file.close()
			__main__.scripteditorfilename = filename
		except IOError, (errno, strerror):
			qt.QMessageBox.critical(self,"Error","<qt>Failed to open script file \"%s\"<br><br>%s</qt>" % (filename,strerror))

	def openFileAs(self):
		import qt
		self.console.clear()
		filename = str( qt.QFileDialog.getOpenFileName(__main__.scripteditorfilename,"*.py;;*", self) )
		if filename == "": return
		self.openFile(filename)

	def saveFile(self):
		try:
			file = open(__main__.scripteditorfilename, "w")
			file.write( str( self.scripttext.text() ) )
			file.close()
		except IOError, (errno, strerror):
			qt.QMessageBox.critical(self,"Error","<qt>Failed to open script file \"%s\"<br><br>%s</qt>" % (__main__.scripteditorfilename,strerror))

	def saveFileAs(self):
		import qt
		filename = str( qt.QFileDialog.getSaveFileName(__main__.scripteditorfilename,"*.py;;*", self) )
		if filename == "": return
		__main__.scripteditorfilename = filename
		self.saveFile()

####################################################################################
# Show the main dialog.

if __name__ == "__main__":
	scriptpath = os.getcwd()
	qtapp = qt.QApplication(sys.argv)
else:
	scriptpath = os.path.dirname(__name__)
	qtapp = qt.qApp

dialog = MainDialog(scriptpath, qtapp.mainWidget())
dialog.exec_loop()



"""
KSpread Kross Python Script.

Export to HTML File.

Description:
This script exports data from KSpread to a HTML file.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

import os, sys

try:
	import qt
except (ImportError):
	raise "Failed to import the required PyQt python module."

class Dialog(qt.QDialog):
	def __init__(self, scriptpath, parent):
		self.scriptpath = scriptpath

		import krosskspreadcore
		self.doc = krosskspreadcore.get("KSpreadDocument")

		import qt
		qt.QDialog.__init__(self, parent, "Dialog", 1, qt.Qt.WDestructiveClose)
		self.setCaption("Export to HTML File")
		layout = qt.QVBoxLayout(self)
		box = qt.QVBox(self)
		box.setMargin(10)
		box.setSpacing(10)
		layout.addWidget(box)

		sheetbox = qt.QHBox(box)
		sheetbox.setSpacing(6)
		sheetlabel = qt.QLabel("Sheet:",sheetbox)
		self.sheetcombo = qt.QComboBox(sheetbox)
		currentsheetname = self.doc.currentSheet().name()
		for sheetname in self.doc.sheetNames():
			self.sheetcombo.insertItem(sheetname)
			if sheetname == currentsheetname:
				self.sheetcombo.setCurrentItem(self.sheetcombo.count() - 1)
		sheetlabel.setBuddy(self.sheetcombo)
		sheetbox.setStretchFactor(self.sheetcombo,1)

		self.styles = {
			"Paper" :
				"html { background-color:#efefef; }"
				"body { background-color:#fafafa; color:#303030; margin:1em; padding:1em; border:#606060 1px solid; }"
			,
			"Plain" :
				"html { background-color:#ffffff; color:#000; }"
				"body { margin:1em; }"
			,
			"Seawater" :
				"html { background-color:#0000aa; }"
				"body { background-color:#000066; color:#efefff; margin:1em; padding:1em; border:#00f 1px solid; }"
				"h1 { color:#0000ff; }"
				"th { color:#6666ff; }"
			,
		}

		stylebox = qt.QHBox(box)
		stylebox.setSpacing(6)
		stylelabel = qt.QLabel("Style:",stylebox)
		self.stylecombo = qt.QComboBox(stylebox)
		stylenames = self.styles.keys()
		stylenames.sort()
		for stylename in stylenames:
			self.stylecombo.insertItem(stylename)
		stylelabel.setBuddy(self.stylecombo)
		stylebox.setStretchFactor(self.stylecombo,1)

		filebox = qt.QHBox(box)
		filebox.setSpacing(6)
		filelabel = qt.QLabel("File:",filebox)
		self.fileedit = qt.QLineEdit(self.getDefaultFile(),filebox)
		btn = qt.QPushButton("...",filebox)
		qt.QObject.connect(btn, qt.SIGNAL("clicked()"),self.browseClicked)
		filelabel.setBuddy(self.fileedit)
		filebox.setStretchFactor(self.fileedit,1)

		btnbox = qt.QHBox(box)
		btnbox.setSpacing(6)
		okbtn = qt.QPushButton(btnbox)
		okbtn.setText("Export")
		okbtn.setDefault(True)
		qt.QObject.connect(okbtn,qt.SIGNAL("clicked()"),self.startExport)
		cancelbtn = qt.QPushButton(btnbox)
		cancelbtn.setText("Cancel")
		qt.QObject.connect(cancelbtn,qt.SIGNAL("clicked()"),self.close)

		box.setMinimumWidth(480)

	def browseClicked(self):
		import qt
		filename = str( qt.QFileDialog.getSaveFileName(str(self.fileedit.text()),"*.htm *.html *.xhtml;;*", self) )
		if filename != "": self.fileedit.setText(filename)
	
	def getDefaultFile(self):
		import os
		try:
			homepath = os.getenv("HOME")
			if not homepath:
				import pwd
				user = os.getenv("USER") or os.getenv("LOGNAME")
				if not user:
					pwent = pwd.getpwuid(os.getuid())
				else:
					pwent = pwd.getpwnam(user)
				homepath = pwent[6]
		except (KeyError, ImportError):
			homepath = os.curdir
		return os.path.join(homepath, "kspreadexport.html")

	def startExport(self):
		import qt

		sheetname = str( self.sheetcombo.currentText() )
		sheet = self.doc.sheetByName( sheetname )
		print "sheetname=%s sheet=%s" % (sheetname,sheet)

		filename = str( self.fileedit.text() )
		try:
			file = open(filename, "w")
		except IOError, (errno, strerror):
			qt.QMessageBox.critical(self,"Error","<qt>Failed to create HTML file \"%s\"<br><br>%s</qt>" % (filename,strerror))
			return

		file.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n")
		file.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\" \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">\n")
		file.write("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n")
		file.write("<head><title>%s</title>\n" % sheetname)

		file.write("<style type=\"text/css\">\n<!--\n")
		file.write( self.styles[ str(self.stylecombo.currentText()) ] )
		file.write("\n//-->\n</style>\n")

		file.write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n")
		file.write("</head><body><h1>%s</h1>\n" % sheetname)

		file.write("<table border=\"1\">\n")
		cell = sheet.firstCell()
		prevrow = -1
		while cell:
			#print "Cell col=%s row=%s value=%s" % (cell.column(),cell.row(),cell.value())

			row = cell.row()
			if row != prevrow:
				prevrow = row
				file.write("<tr>")
				file.write("<th>%s</th>" % row)

			file.write("<td>%s</td>" % cell.value())

			cell = cell.nextCell()
			if cell == None or cell.row() != prevrow:
				file.write("</tr>\n")

		file.write("</table>\n")
		file.write("</body></html>\n")

		file.close()
		self.close()

if __name__ == "__main__":
	scriptpath = os.getcwd()
	qtapp = qt.QApplication(sys.argv)
else:
	scriptpath = os.path.dirname(__name__)
	qtapp = qt.qApp

dialog = Dialog(scriptpath, qtapp.mainWidget())
dialog.exec_loop()



""" 
Python script for a GUI-dialog.

Description:
Python script to provide an abstract GUI for other python scripts. That
way we've all the GUI-related code within one single file and are
able to easily modify GUI-stuff in a central place.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Published as-is without any warranties.
"""

def getHome():
	""" Return the homedirectory. """
	import os
	try:
		home = os.getenv("HOME")
		if not home:
			import pwd
			user = os.getenv("USER") or os.getenv("LOGNAME")
			if not user:
				pwent = pwd.getpwuid(os.getuid())
			else:
				pwent = pwd.getpwnam(user)
			home = pwent[6]
		return home
	except (KeyError, ImportError):
		return os.curdir

class TkDialog:
	""" This class is used to wrap Tkinter into a more abstract interface."""

	def __init__(self, title):
		import Tkinter
		self.root = Tkinter.Tk()
		self.root.title(title)
		self.root.deiconify()

		mainframe = self.Frame(self, self.root)
		self.widget = mainframe.widget

	class Widget:
		def __init__(self, dialog, parent):
			self.dialog = dialog
			self.parent = parent
		#def setVisible(self, visibled): pass
		#def setEnabled(self, enabled): pass

	class Frame(Widget):
		def __init__(self, dialog, parent):
			#TkDialog.Widget.__init__(self, dialog, parent)
			import Tkinter
			self.widget = Tkinter.Frame(parent)
			self.widget.pack()

	class Label(Widget):
		def __init__(self, dialog, parent, caption):
			#TkDialog.Widget.__init__(self, dialog, parent)
			import Tkinter
			self.widget = Tkinter.Label(parent, text=caption)
			self.widget.pack(side=Tkinter.TOP)

	class CheckBox(Widget):
		def __init__(self, dialog, parent, caption, checked = True):
			#TkDialog.Widget.__init__(self, dialog, parent)
			import Tkinter
			self.checkstate = Tkinter.IntVar()
			self.checkstate.set(checked)
			self.widget = Tkinter.Checkbutton(parent, text=caption, variable=self.checkstate)
			self.widget.pack(side=Tkinter.TOP)
		def isChecked(self):
			return self.checkstate.get()

	class List(Widget):
		def __init__(self, dialog, parent, caption, items):
			#TkDialog.Widget.__init__(self, dialog, parent)
			import Tkinter

			listframe = Tkinter.Frame(parent)
			listframe.pack()

			Tkinter.Label(listframe, text=caption).pack(side=Tkinter.LEFT)

			self.items = items
			self.variable = Tkinter.StringVar()
			itemlist = apply(Tkinter.OptionMenu, (listframe, self.variable) + tuple( items ))
			itemlist.pack(side=Tkinter.LEFT)
		def get(self):
			return self.variable.get()
		def set(self, index):
			self.variable.set( self.items[index] )

	class Button(Widget):
		def __init__(self, dialog, parent, caption, commandmethod):
			#TkDialog.Widget.__init__(self, dialog, parent)
			import Tkinter
			self.widget = Tkinter.Button(parent, text=caption, command=self.doCommand)
			self.commandmethod = commandmethod
			self.widget.pack(side=Tkinter.LEFT)
		def doCommand(self):
			try:
				self.commandmethod()
			except:
				#TODO why the heck we arn't able to redirect exceptions?
				import traceback
				import StringIO
				fp = StringIO.StringIO()
				traceback.print_exc(file=fp)
				import tkMessageBox
				tkMessageBox.showerror("Exception", fp.getvalue())
				#self.dialog.root.destroy()

	class Edit(Widget):
		def __init__(self, dialog, parent, caption, text):
			#TkDialog.Widget.__init__(self, dialog, parent)
			import Tkinter
			self.widget = Tkinter.Frame(parent)
			self.widget.pack()
			label = Tkinter.Label(self.widget, text=caption)
			label.pack(side=Tkinter.LEFT)
			self.entrytext = Tkinter.StringVar()
			self.entrytext.set(text)
			self.entry = Tkinter.Entry(self.widget, width=36, textvariable=self.entrytext)
			self.entry.pack(side=Tkinter.LEFT)
		def get(self):
			return self.entrytext.get()

	class FileChooser(Edit):
		def __init__(self, dialog, parent, caption, initialfile = None, filetypes = None):
			TkDialog.Edit.__init__(self, dialog, parent, caption, initialfile)
			import Tkinter

			self.initialfile = initialfile
			self.entrytext.set(initialfile)

			btn = Tkinter.Button(self.widget, text="...", command=self.browse)
			btn.pack(side=Tkinter.LEFT)

			if filetypes:
				self.filetypes = filetypes
			else:
				self.filetypes = (('All files', '*'),)

		def browse(self):
			import os
			text = self.entrytext.get()
			d = os.path.dirname(text) or os.path.dirname(self.initialfile)
			f = os.path.basename(text) or os.path.basename(self.initialfile)

			import tkFileDialog
			file = tkFileDialog.asksaveasfilename(
					   initialdir=d,
					   initialfile=f,
					   #defaultextension='.html',
					   filetypes=self.filetypes
			)
			if file:
				self.entrytext.set( file )

	class MessageBox:
		def __init__(self, dialog, typename, caption, message):
			self.widget = dialog.widget
			self.typename = typename
			self.caption = str(caption)
			self.message = str(message)
		def show(self):
			import tkMessageBox
			if self.typename == "okcancel":
				return tkMessageBox.askokcancel(self.caption, self.message,icon=tkmessageBox.QESTION)
			else:
				tkMessageBox.showinfo(self.caption, self.message)
			return True

	def show(self):
		self.root.mainloop()
		
	def close(self):
		self.root.destroy()

class QtDialog:
	""" This class is used to wrap pyQt/pyKDE into a more abstract interface."""

	def __init__(self, title):
		import qt
		
		class Dialog(qt.QDialog):
			def __init__(self, parent = None, name = None, modal = 0, fl = 0):
				qt.QDialog.__init__(self, parent, name, modal, fl)
				qt.QDialog.accept = self.accept
				self.layout = qt.QVBoxLayout(self)
				self.layout.setSpacing(6)
				self.layout.setMargin(11)
				
		class Label(qt.QLabel):
			def __init__(self, dialog, parent, caption):
				qt.QLabel.__init__(self, parent)
				self.setText("<qt>%s</qt>" % caption.replace("\n","<br>"))
				
		class Frame(qt.QHBox):
			def __init__(self, dialog, parent):
				qt.QHBox.__init__(self, parent)
				self.widget = self
				self.setSpacing(6)

		class Edit(qt.QHBox):
			def __init__(self, dialog, parent, caption, text):
				qt.QHBox.__init__(self, parent)
				self.setSpacing(6)
				label = qt.QLabel(caption, self)
				self.edit = qt.QLineEdit(self)
				self.edit.setText( str(text) )
				self.setStretchFactor(self.edit, 1)
				label.setBuddy(self.edit)
			def get(self):
				return self.edit.text()

		class Button(qt.QPushButton):
			#def __init__(self, *args):
			def __init__(self, dialog, parent, caption, commandmethod):
				#apply(qt.QPushButton.__init__, (self,) + args)
				qt.QPushButton.__init__(self, parent)
				self.commandmethod = commandmethod
				self.setText(caption)
				qt.QObject.connect(self, qt.SIGNAL("clicked()"), self.commandmethod)


		class CheckBox(qt.QCheckBox):
			def __init__(self, dialog, parent, caption, checked = True):
				#TkDialog.Widget.__init__(self, dialog, parent)
				qt.QCheckBox.__init__(self, parent)
				self.setText(caption)
				self.setChecked(checked)
			#def isChecked(self):
			#	return self.isChecked()

		class List(qt.QHBox):
			def __init__(self, dialog, parent, caption, items):
				qt.QHBox.__init__(self, parent)
				self.setSpacing(6)
				label = qt.QLabel(caption, self)
				self.combo = qt.QComboBox(self)
				self.setStretchFactor(self.combo, 1)
				label.setBuddy(self.combo)
				for item in items:
					self.combo.insertItem( str(item) )
			def get(self):
				return self.combo.currentText()
			def set(self, index):
				self.combo.setCurrentItem(index)

		class FileChooser(qt.QHBox):
			def __init__(self, dialog, parent, caption, initialfile = None, filetypes = None):
				#apply(qt.QHBox.__init__, (self,) + args)
				qt.QHBox.__init__(self, parent)
				self.setMinimumWidth(400)

				self.initialfile = initialfile
				self.filetypes = filetypes
				
                                self.setSpacing(6)
				label = qt.QLabel(caption, self)
				self.edit = qt.QLineEdit(self)
				self.edit.setText(self.initialfile)
				self.setStretchFactor(self.edit, 1)
				label.setBuddy(self.edit)
				
				browsebutton = Button(dialog, self, "...", self.browseButtonClicked)
				#qt.QObject.connect(browsebutton, qt.SIGNAL("clicked()"), self.browseButtonClicked)

			def get(self):
				return self.edit.text()

			def browseButtonClicked(self):
				filtermask = ""
				import types
				if isinstance(self.filetypes, types.TupleType):
					for ft in self.filetypes:
						if len(ft) == 1:
							filtermask += "%s\n" % (ft[0])
						if len(ft) == 2:
							filtermask += "%s|%s (%s)\n" % (ft[1],ft[0],ft[1])
				if filtermask == "":
					filtermask = "All files (*.*)"
				else:
					filtermask = filtermask[:-1]
					
				filename = None
				try:
					print "QtDialog.FileChooser.browseButtonClicked() kfile.KFileDialog"
					# try to use the kfile module included in pykde
					import kfile
					filename = kfile.KFileDialog.getOpenFileName(self.initialfile, filtermask, self, "Save to file")
				except:
					print "QtDialog.FileChooser.browseButtonClicked() qt.QFileDialog"
					# fallback to Qt filedialog
					filename = qt.QFileDialog.getOpenFileName(self.initialfile, filtermask, self, "Save to file")
				if filename != None and filename != "":
					self.edit.setText(filename)
					
		class MessageBox:
			def __init__(self, dialog, typename, caption, message):
				self.widget = dialog.widget
				self.typename = typename
				self.caption = str(caption)
				self.message = str(message)
			def show(self):
				result = 1
				if self.typename == "okcancel":
					result = qt.QMessageBox.question(self.widget, self.caption, self.message, "&Ok", "&Cancel", "", 1)
				else:
					qt.QMessageBox.information(self.widget, self.caption, self.message, "&Ok")
					result = 0
				if result == 0:
					return True
				return False

		self.app = qt.qApp
		self.dialog = Dialog(self.app.mainWidget(), "Dialog", 1, qt.Qt.WDestructiveClose)
		self.dialog.setCaption(title)

		self.widget = qt.QVBox(self.dialog)
		self.widget.setSpacing(6)
		self.dialog.layout.addWidget(self.widget)

		self.Frame = Frame
		self.Label = Label
		self.Edit = Edit
		self.Button = Button
 		self.CheckBox = CheckBox
 		self.List = List
		self.FileChooser = FileChooser
		self.MessageBox = MessageBox
		
	def show(self):
		import qt
		qt.QApplication.setOverrideCursor(qt.Qt.arrowCursor)
		self.dialog.exec_loop()
		qt.QApplication.restoreOverrideCursor()

	def close(self):
		print "QtDialog.close()"
		self.dialog.close()
		#self.dialog.deleteLater()

class Dialog:
	""" Central class that provides abstract GUI-access to the outer world. """

	def __init__(self, title):
		self.dialog = None

		try:
			print "Trying to import PyQt..."
			self.dialog = QtDialog(title)
			print "PyQt is our toolkit!"
		except:
			try:
				print "Failed to import PyQt. Trying to import TkInter..."
				self.dialog = TkDialog(title)
				print "Falling back to TkInter as our toolkit!"
			except:
				raise "Failed to import GUI-toolkit. Please install the PyQt or the Tkinter python module."

                self.widget = self.dialog.widget

        def show(self):
		self.dialog.show()

	def close(self):
		self.dialog.close()

	def addFrame(self, parentwidget):
		return self.dialog.Frame(self.dialog, parentwidget.widget)

	def addLabel(self, parentwidget, caption):
		return self.dialog.Label(self.dialog, parentwidget.widget, caption)

	def addCheckBox(self, parentwidget, caption, checked = True):
		return self.dialog.CheckBox(self.dialog, parentwidget.widget, caption, checked)

	def addButton(self, parentwidget, caption, commandmethod):
		return self.dialog.Button(self.dialog, parentwidget.widget, caption, commandmethod)

	def addEdit(self, parentwidget, caption, text):
		return self.dialog.Edit(self.dialog, parentwidget.widget, caption, text)

	def addFileChooser(self, parentwidget, caption, initialfile = None, filetypes = None):
		return self.dialog.FileChooser(self.dialog, parentwidget.widget, caption, initialfile, filetypes)

	def addList(self, parentwidget, caption, items):
		return self.dialog.List(self.dialog, parentwidget.widget, caption, items)
		
	def showMessageBox(self, typename, caption, message):
		return self.dialog.MessageBox(self.dialog, typename, caption, message)



"""
CopyCenterPlugin to provide 'KexiDB'.

Description:
This python-script is a plugin for the CopyCenter.py.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
GPL v2 or higher.
"""

class CopyCenterPlugin:
	""" The CopyCenterPlugin to provide abstract access to the 'KexiDB'
	framework to CopyCenter.py """

	name = "Kexi Database"
	""" The name this plugin has. The name should be unique and
	will be used for displaying a caption. """

	class Plugin:
		""" The implementation of a plugin which is published to the
		CopyCenter.py script. While there exists only one instance of
		the CopyCenterPlugin class, there will be n instances of this
		Plugin class (one for 'source' aka read-data-from and one for
		'destination' aka write-data-to) created from within the
		CopyCenter.py. The Source and Destination classes are extending
		this Plugin class with specialized functionality. """
		def __init__(self,copycenterplugin):
			""" Constructor. """
			self.copycenterplugin = copycenterplugin
			self.widget = None
			self.options = {
				'autoconnect' : False,
				'project' : '', #'~/test.kexi',
				'driver' : '', #'MySQL', #'SQLite3','MySQL',...
				'file' : '', #'/path/to/mysqlite3dbfile.kexi'
				'hostname' : '127.0.0.1',
				'port' : '',
				'usesocketfile' : False,
				'socketfile' : '',
				'username' : '',
				'password' : '',
				'database' : '',
				'table' : '',
				'fields' : '',
				'where' : '',
			}
			self.connection = copycenterplugin.Connection(self)
		def isFinished(self):
			return self.connection.isFinished()
		def finish(self):
			""" Called if reading is finished."""
			self.connection.finish()
		def createWidget(self, dialog, parent):
			""" Create and return a widget to modify the plugin settings. """
			return self.copycenterplugin.createWidget(dialog, self, parent)

	class Source(Plugin):
		""" Specialization of the Plugin class to implement the
		'source' aka read-data-from functionality. """
		plugintype = "Source"
		def init(self,copierer):
			""" Called if reading should be initialize. """
			self.connection.init(copierer)
			self.connection.initRead()
			# Called if a record should be readed.
			self.read = self.connection.readRecord

	class Destination(Plugin):
		""" Specialization of the Plugin class to implement the
		'destination' aka write-data-to functionality. """
		plugintype = "Destination"
		def init(self,copierer):
			""" Called if writing should be initialize. """
			self.connection.init(copierer)
			self.connection.initWrite()
			# Called if a record should be written.
			self.write = self.connection.writeRecord

	class Connection:
		""" Abstract access to work with KexiDB. """
		def __init__(self,plugin):
			self.plugin = plugin
			self.copierer = None
			self.kexidbconnection = None
			self.kexidbcursor = None
			self.tableschema = None
			self.fieldlist = None
		def lastError(self): return self.kexidbconnection.lastError()
		def connect(self): return self.kexidbconnection.connect()
		def disconnect(self):
			if self.kexidbconnection == None or self.kexidbconnection.disconnect():
				self.kexidbconnection = None
				return True
			return False
		def isConnected(self): return self.kexidbconnection != None and self.kexidbconnection.isConnected()
		def tableNames(self): return self.kexidbconnection.tableNames()
		def hasTableName(self,tablename): return tablename in self.kexidbconnection.tableNames()
		def tableSchema(self,tablename): return self.kexidbconnection.tableSchema(tablename)
		
		def init(self,copierer):
			self.copierer = copierer
			if self.kexidbconnection == None:
				if self.plugin.widget == None:
					raise "No connection established."
				self.copierer.appendProgressMessage("<i>Trying to connect...</i>")
				if self.plugin.widget.driverbox.driver == None:
					raise "Invalid driver."
				if not self.plugin.widget.connectClicked():
					raise "Failed to connect."
			connectiondata = self.kexidbconnection.data()
			self.copierer.appendProgressMessage("Connected: %s %s" % (connectiondata.driverName(),connectiondata.serverInfoString()))

			tablename = str(self.plugin.widget.tablebox.tableedit.text())
			if tablename == "":
				raise "No table defined"
			fields = [ f.strip() for f in str(self.plugin.widget.fieldbox.fieldsedit.text()).split(",") if len(f) > 0 ]
			if len(fields) < 1:
				raise "No fields defined"

			self.tableschema = self.kexidbconnection.tableSchema(tablename)
			if not self.tableschema: raise "No such tableschema \"%s\"" % tablename
			self.copierer.appendProgressMessage("Table: %s" % self.tableschema.name())

			if len(fields) == 1 and fields[0] == "*":
				self.fieldlist = self.tableschema.fieldlist()
			else:
				self.fieldlist = self.tableschema.fieldlist().subList(fields)
			if not self.fieldlist: raise "No such fields \"%s\"" % fields
			fieldlistnames = self.fieldlist.names()
			if len(fieldlistnames) < 1: raise "No valid fields defined for \"%s\"" % fields
			self.copierer.appendProgressMessage("Fields: %s" % fieldlistnames)
		def finish(self):
			if self.plugin.widget == None:
				self.disconnect()
			else:
				self.plugin.widget.disconnectClicked()
			self.kexidbcursor = None
			self.kexidbconnection = None
			self.tableschema = None
			self.fieldlist = None
			self.copierer = None
		def isFinished(self):
			return self.copierer == None

		def initRead(self):
			print "Initialize read"
			#queryschema = self.plugin.copycenterplugin.drivermanager.querySchema()
			queryschema = self.tableschema.query()
			queryschema.fieldlist().setFields(self.fieldlist)
			print "QuerySchema: %s" % queryschema.fieldlist().names()

			whereexpression = str(self.plugin.widget.whereedit.text())
			if whereexpression != "":
				print "WHERE-expression: %s" % whereexpression
				if not queryschema.setWhereExpression(whereexpression):
					raise "Invalid WHERE-expression."

			#print "QuerySchema statement=%s" % queryschema.statement()
			self.kexidbcursor = self.kexidbconnection.executeQuerySchema(queryschema)
			if not self.kexidbcursor:
				raise "Failed to create cursor."
			if not self.kexidbcursor.moveFirst():
				raise "The cursor has no records to read from."

		def readRecord(self):
			if self.kexidbcursor == None or self.kexidbcursor.eof():
				return None
			record = []
			for i in range( self.kexidbcursor.fieldCount() ):
				record.append( self.kexidbcursor.value(i) )
			self.kexidbcursor.moveNext()
			#print "read record: %s" % record
			return record

		def initWrite(self):
			print "Initialize write"

		def writeRecord(self,record):
			print "write record: %s" % record
			if self.kexidbconnection.insertRecord(self.fieldlist,record):
				print "=> insert successfully"
				self.copierer.writeSuccess(record, 1)
			else:
				print "=> insert failed: %s" % self.kexidbconnection.lastError()
  				self.copierer.writeFailed(record)
			#import time
  			#time.sleep(1)
  			return True

	def __init__(self, copycenter):
		""" Constructor.  """
		import krosskexidb
		self.drivermanager = krosskexidb.DriverManager()
		self.copycenter = copycenter

	def createWidget(self, dialog, plugin, parent):
		""" Each plugin may provide a qt.QWidget back to the
		CopyCenter.py. The widget will be used to configure our
		plugin settings. """

		import qt
		import os
		import re

		self.dialog = dialog
		self.mainbox = None
		class ProjectBox(qt.QHBox):
			def __init__(self,main,copycenterplugin,plugin,parent):
				self.main = main
				self.copycenterplugin = copycenterplugin
				self.plugin = plugin

				qt.QHBox.__init__(self,parent)
				prjlabel = qt.QLabel("Project File:",self)
				self.prjcombo = qt.QComboBox(self)
				self.prjcombo.setEditable(True)
				self.prjcombo.insertItem("")

				path = copycenterplugin.copycenter.homepath
				for f in os.listdir(path):
					file = os.path.join(path,f)
					if os.path.isfile(file) and re.search(".+\\.(kexi|kexis|kexic)$",f):
						self.prjcombo.insertItem(os.path.join("~",f))
						
				prjlabel.setBuddy(self.prjcombo)
				prjsavebtn = qt.QPushButton("...",self)
				qt.QObject.connect(prjsavebtn, qt.SIGNAL("clicked()"),self.buttonClicked)
				qt.QObject.connect(self.prjcombo, qt.SIGNAL("textChanged(const QString&)"), self.main.projectChanged)
				self.setStretchFactor(self.prjcombo,1)
			def buttonClicked(self):
				text = str(self.prjcombo.currentText())
				if text == "":
					text = self.copycenterplugin.copycenter.homepath
				elif re.search("^\\~(\\/|\\\\)",text):
					import os
					text = os.path.join(self.copycenterplugin.copycenter.homepath,text[2:])
				if self.plugin.plugintype == "Source":
					filename = qt.QFileDialog.getOpenFileName(text,"*.kexi *.kexis *.kexic;;*",self.copycenterplugin.dialog)
				else: # "Destination":
					filename = qt.QFileDialog.getSaveFileName(text,"*.kexi *.kexis *.kexic;;*",self.copycenterplugin.dialog)
				if str(filename) != "": self.prjcombo.setCurrentText(str(filename))

		class DriverBox(qt.QVBox):
			def __init__(self,main,parent):
				qt.QVBox.__init__(self,parent)
				self.main = main
				self.copycenterplugin = main.copycenterplugin
				self.plugin = main.plugin
				self.driver = None

				driverbox = qt.QHBox(self)
				driverlabel = qt.QLabel("Driver:",driverbox)
				self.drivercombo = qt.QComboBox(driverbox)
				self.drivercombo.insertItem("")
				for driver in self.copycenterplugin.drivermanager.driverNames():
					self.drivercombo.insertItem(driver)

				qt.QObject.connect(self.drivercombo, qt.SIGNAL("activated(int)"), self.activated)
				driverlabel.setBuddy(self.drivercombo)
				driverbox.setStretchFactor(self.drivercombo,1)
				
				self.box = qt.QVBox(self)
				self.mainbox = None

			def activated(self,index):
				drivertext = str(self.drivercombo.currentText())

				self.box.hide()
				if self.mainbox:
					self.mainbox.hide()
					self.mainbox.destroy()
					self.mainbox = None
				if index == 0 or drivertext == "":
					self.driver = None
					self.box.show()
					self.main.updateConnectButtons()
					return False

				self.driver = self.copycenterplugin.drivermanager.driver(drivertext)

				mainbox = qt.QVBox(self.box)
				mainbox.setSpacing(2)

				if self.driver.isFileDriver():
					filebox = qt.QHBox(mainbox)
					filelabel = qt.QLabel("File:",filebox)
					self.fileedit = qt.QLineEdit(self.plugin.options['file'],filebox)
					filelabel.setBuddy(self.fileedit)
					filebox.setStretchFactor(self.fileedit,1)
					filebtn = qt.QPushButton("...",filebox)
					qt.QObject.connect(filebtn, qt.SIGNAL("clicked()"), self.fileClicked)
				else:
					hostbox = qt.QHBox(mainbox)
					hostlabel = qt.QLabel("Hostname:",hostbox)
					self.hostedit = qt.QLineEdit(self.plugin.options['hostname'],hostbox)
					hostlabel.setBuddy(self.hostedit)
					hostbox.setStretchFactor(self.hostedit,1)

					portbox = qt.QHBox(mainbox)
					portlabel = qt.QLabel("Port:",portbox)
					self.portedit = qt.QLineEdit(self.plugin.options['port'],portbox)
					portlabel.setBuddy(self.portedit)
					portbox.setStretchFactor(self.portedit,1)

					sockbox = qt.QHBox(mainbox)
					self.sockfilecheckbox = qt.QCheckBox("Socket File:",sockbox)
					qt.QObject.connect(self.sockfilecheckbox, qt.SIGNAL("toggled(bool)"), self.sockfilecheckboxClicked)
					self.sockfilebox = qt.QHBox(sockbox)
					self.sockfileedit = qt.QLineEdit(self.plugin.options['socketfile'],self.sockfilebox)
					self.sockfilebox.setEnabled(False)
					sockfilebtn = qt.QPushButton("...",self.sockfilebox)
					self.sockfilecheckbox.setChecked( str(self.plugin.options['usesocketfile']) == str(True) )
					qt.QObject.connect(sockfilebtn, qt.SIGNAL("clicked()"), self.sockfileClicked)
					self.sockfilebox.setStretchFactor(self.sockfileedit,1)
					sockbox.setStretchFactor(self.sockfilebox,1)

					userbox = qt.QHBox(mainbox)
					userlabel = qt.QLabel("Username:",userbox)
					self.useredit = qt.QLineEdit(self.plugin.options['username'],userbox)
					userlabel.setBuddy(self.useredit)
					userbox.setStretchFactor(self.useredit,1)

					passbox = qt.QHBox(mainbox)
					passlabel = qt.QLabel("Password:",passbox)
					self.passedit = qt.QLineEdit(self.plugin.options['password'],passbox)
					self.passedit.setEchoMode(qt.QLineEdit.Password)
					passlabel.setBuddy(self.passedit)
					passbox.setStretchFactor(self.passedit,1)

					dbbox = qt.QHBox(mainbox)
					dblabel = qt.QLabel("Database:",dbbox)
					self.dbedit = qt.QLineEdit(self.plugin.options['database'],dbbox)
					dblabel.setBuddy(self.dbedit)
					dbbox.setStretchFactor(self.dbedit,1)
				#self.tablecombo.setText("")

				self.mainbox = mainbox
				self.mainbox.show()
				self.box.show()
				self.main.updateConnectButtons()
				return True

			def fileClicked(self):
				text = str(self.fileedit.text())
				if text == "": text = self.copycenterplugin.copycenter.homepath
				if self.plugin.plugintype == "Source":
					filename = qt.QFileDialog.getOpenFileName(text,"*",self.copycenterplugin.dialog)
				else: # "Destination":
					filename = qt.QFileDialog.getSaveFileName(text,"*",self.copycenterplugin.dialog)
				if str(filename) != "": self.fileedit.setText(str(filename))
			def sockfilecheckboxClicked(self,checked):
				self.sockfilebox.setEnabled(checked)
				
			def sockfileClicked(self):
				text = str(self.sockfileedit.text())
				if text == "": text = self.copycenterplugin.copycenter.homepath
				if self.plugin.plugintype == "Source":
					filename = qt.QFileDialog.getOpenFileName(text,"*",self.copycenterplugin.dialog)
				else: # "Destination":
					filename = qt.QFileDialog.getSaveFileName(text,"*",self.copycenterplugin.dialog)
				if str(filename) != "": self.sockfileedit.setText(str(filename))

		class TableBox(qt.QHBox):
			def __init__(self,copycenterplugin,plugin,parent):
				qt.QHBox.__init__(self,parent)
				self.copycenterplugin = copycenterplugin
				self.plugin = plugin
				tablelabel = qt.QLabel("Table:",self)
				self.tableedit = qt.QLineEdit(self.plugin.options['table'],self)
				self.tablebtn = qt.QPushButton("...",self)
				self.tablebtn.setEnabled(False)
				qt.QObject.connect(self.tablebtn, qt.SIGNAL("clicked()"), self.buttonClicked)
				tablelabel.setBuddy(self.tableedit)
				self.setStretchFactor(self.tableedit,1)
			def buttonClicked(self):
				ListViewDialog = self.copycenterplugin.dialog.ListViewDialog
				class TableDialog(ListViewDialog):
					def __init__(self,tablebox):
						ListViewDialog.__init__(self,tablebox,"Tables")
						self.mainwidget = tablebox
						self.listview.addColumn("Name")
						text = str(self.mainwidget.tableedit.text())
						item = None
						for table in self.mainwidget.plugin.connection.tableNames():
							if item == None:
								item = qt.QListViewItem(self.listview,table)
							else:
								item = qt.QListViewItem(self.listview,item,table)
							if table == text:
								self.listview.setSelected(item,True)
								self.listview.ensureItemVisible(item)
						qt.QObject.connect(self.listview, qt.SIGNAL("doubleClicked(QListViewItem*, const QPoint&, int)"), self.okClicked)
						qt.QObject.connect(self.okbtn, qt.SIGNAL("clicked()"), self.okClicked)
					def okClicked(self):
						item = self.listview.selectedItem()
						if item == None:
							self.mainwidget.tableedit.setText("")
						else:
							self.mainwidget.tableedit.setText(item.text(0))
						self.close()
				dialog = TableDialog(self)
				dialog.show()

		class FieldBox(qt.QHBox):
			def __init__(self,copycenterplugin,plugin,parent):
				qt.QHBox.__init__(self,parent)
				self.copycenterplugin = copycenterplugin
				self.plugin = plugin
				self.tablename = ""
				fieldslabel = qt.QLabel("Fields:",self)
				self.fieldsedit = qt.QLineEdit(self.plugin.options['fields'],self)
				self.setStretchFactor(self.fieldsedit,1)
				fieldslabel.setBuddy(self.fieldsedit)
				self.fieldsbtn = qt.QPushButton("...",self)
				self.fieldsbtn.setEnabled(False)
				qt.QObject.connect(self.fieldsbtn, qt.SIGNAL("clicked()"), self.fieldsClicked)
			def fieldsClicked(self):
				ListViewDialog = self.copycenterplugin.dialog.ListViewDialog
				class FieldsDialog(ListViewDialog):
					def __init__(self, fieldbox):
						ListViewDialog.__init__(self,fieldbox,"Fields")
						self.fieldbox = fieldbox
						self.listview.setSelectionMode(qt.QListView.Multi)
						self.listview.setSorting(-1)
						self.listview.header().setClickEnabled(False)
						self.listview.addColumn("Name")
						self.listview.addColumn("Type")
						self.listview.addColumn("Options")
						fieldslist = str(self.fieldbox.fieldsedit.text()).split(",")
						allfields = ("*" in fieldslist)
						tableschema = self.fieldbox.plugin.connection.tableSchema(self.fieldbox.tablename)
						item = None
						for field in tableschema.fieldlist().fields():
							opts = []
							for opt in ("isAutoInc","isNotNull","isNotEmpty"):
								if getattr(field,opt)():
									opts.append(opt[2:])
							item = self.addItem(( field.name(),field.type(),",".join(opts) ),item)
							if allfields or field.name() in fieldslist:
								self.listview.setSelected(item,True)
						qt.QObject.connect(self.okbtn, qt.SIGNAL("clicked()"), self.okClicked)
					def okClicked(self):
						selitems = []
						item = self.listview.firstChild()
						while item:
							if item.isSelected():
								selitems.append(str(item.text(0)))
							item = item.nextSibling()
						self.fieldbox.fieldsedit.setText(",".join(selitems))
						self.close()
				dialog = FieldsDialog(self)
				dialog.show()
			def tableChanged(self, text):
				self.tablename = str(text)
				if self.plugin.connection.isConnected():
					if self.plugin.connection.hasTableName(self.tablename):
						self.fieldsbtn.setEnabled(True)
						return
				self.fieldsbtn.setEnabled(False)

		class MainBox(qt.QHBox):
			def __init__(self,copycenterplugin,plugin,parent):
				qt.QHBox.__init__(self,parent)
				self.copycenterplugin = copycenterplugin
				self.plugin = plugin

				self.prjbox = ProjectBox(self,copycenterplugin,plugin,parent)
				self.driverbox = DriverBox(self,parent)

				statusbar = qt.QHBox(parent)
				statusbar.setSpacing(2)
				#self.statuslabel = qt.QLabel("Disconnected",statusbar)
				#statusbar.setStretchFactor(self.statuslabel,1)
				statusbar.setStretchFactor(qt.QWidget(statusbar),1)
				self.connectbtn = qt.QPushButton("Connect",statusbar)
				self.connectbtn.setEnabled(False)
				qt.QObject.connect(self.connectbtn, qt.SIGNAL("clicked()"),self.connectClicked)
				self.disconnectbtn = qt.QPushButton("Disconnect",statusbar)
				self.disconnectbtn.setEnabled(False)
				qt.QObject.connect(self.disconnectbtn, qt.SIGNAL("clicked()"),self.disconnectClicked)

				#self.connectionbox = ConnectionBox(copycenterplugin,plugin,parent)
				self.tablebox = TableBox(copycenterplugin,plugin,parent)
				self.fieldbox = FieldBox(copycenterplugin,plugin,parent)
				qt.QObject.connect(self.tablebox.tableedit, qt.SIGNAL("textChanged(const QString&)"), self.fieldbox.tableChanged)

				if self.plugin.options['project'] != '':
					self.prjbox.prjcombo.setCurrentText(self.plugin.options['project'])

				if self.plugin.options['driver'] != '':
					try:
						item = str(self.driverbox.drivercombo.listBox().findItem(self.plugin.options['driver'],qt.Qt.ExactMatch).text())
						self.driverbox.drivercombo.setCurrentText(item)
						self.driverbox.activated(item)
					except:
						pass

				if self.plugin.plugintype == "Destination":
					#typebox = qt.QHBox(parent)
					#label = qt.QLabel("Operation:",typebox)
					#combobox = qt.QComboBox(typebox)
					#combobox.insertItem("Append")
					#combobox.insertItem("Replace")
					#combobox.insertItem("Update")
					#combobox.insertItem("Update/Insert")
					#combobox.insertItem("Insert new")
					#label.setBuddy(combobox)
					#typebox.setStretchFactor(combobox,1)
					pass
				elif self.plugin.plugintype == "Source":
					wherebox = qt.QHBox(parent)
					wherelabel = qt.QLabel("Where:",wherebox)
					self.whereedit = qt.QLineEdit(self.plugin.options['where'],wherebox)

					#orderbox = qt.QHBox(parent)
					#orderlabel = qt.QLabel("Order By:",orderbox)
					#orderedit = qt.QLineEdit("",orderbox)

				#errbox = qt.QHBox(parent)
				#errlabel = qt.QLabel("On Error:",errbox)
				#errcombo = qt.QComboBox(errbox)
				#errcombo.insertItem("Ask")
				#errcombo.insertItem("Skip")
				#errcombo.insertItem("Abort")
				#errlabel.setBuddy(errcombo)
				#errbox.setStretchFactor(errcombo,1)

				if self.plugin.options['autoconnect']:
					self.connectClicked()

			def projectChanged(self, text):
				#if self.driverbox.drivercombo.currentItem() != 0:
				#	self.driverbox.drivercombo.setCurrentItem(0)

				file = str(text)
				import os
				if re.search("^\\~(\\/|\\\\)",file):
					file = os.path.join(self.copycenterplugin.copycenter.homepath,file[2:])
				if file == "" or not os.path.isfile(file):
					self.driverbox.drivercombo.setCurrentItem(0)
					self.driverbox.activated(0)
					return

				connectiondata = self.copycenterplugin.drivermanager.createConnectionDataByFile(file)
				if connectiondata == None:
					raise "Unsupported file."

				drivername = connectiondata.driverName().lower()
				print "driver: %s" % drivername
				for i in range(1,self.driverbox.drivercombo.count()):
					if drivername == self.driverbox.drivercombo.text(i).lower():
						self.driverbox.drivercombo.setCurrentItem(i)
						self.driverbox.activated(i)
						break

				if self.driverbox.driver != None:
					if self.driverbox.driver.isFileDriver():
						self.driverbox.fileedit.setText(connectiondata.fileName())
					else: # server
						self.driverbox.hostedit.setText(connectiondata.hostName())
						self.driverbox.portedit.setText(str(connectiondata.port()))
						self.driverbox.sockfilecheckbox.setChecked(connectiondata.localSocketFileUsed())
						self.driverbox.sockfileedit.setText(connectiondata.localSocketFileName())
						self.driverbox.useredit.setText(connectiondata.userName())
						self.driverbox.passedit.setText(connectiondata.password())
						self.driverbox.dbedit.setText(connectiondata.databaseName())

			def connectClicked(self):
				if self.driverbox.driver == None:
					print "No driver selected."
					return False
				connectiondata = self.copycenterplugin.drivermanager.createConnectionData()
				if self.driverbox.driver.isFileDriver():
					file = str(self.driverbox.fileedit.text())
					if file == "" or not os.path.isfile(file):
						qt.QMessageBox.critical(self,"Failed to connect","There exists no such database file \"%s\"" % file)
						return False
					connectiondata.setFileName(file)
					connectiondata.setDatabaseName(file)
				else:
					connectiondata.setHostName(str(self.driverbox.hostedit.text()))
					connectiondata.setPort(str(self.driverbox.portedit.text()))
					connectiondata.setLocalSocketFileUsed(self.driverbox.sockfilecheckbox.isChecked())
					connectiondata.setLocalSocketFileName(str(self.driverbox.sockfileedit.text()))
					connectiondata.setPassword(str(self.driverbox.passedit.text()))
					connectiondata.setUserName(str(self.driverbox.useredit.text()))
					connectiondata.setDatabaseName(str(self.driverbox.dbedit.text()))
				print "Creating connection"
				connection = self.driverbox.driver.createConnection(connectiondata)
				print "Trying to connect"
				if not connection.connect():
					qt.QMessageBox.critical(self,"Failed to connect",connection.lastError())
					return False
				print "Use database \"%s\"" % connectiondata.databaseName()
				if not connection.useDatabase( connectiondata.databaseName() ):
					qt.QMessageBox.critical(self,"Failed to connect",connection.lastError())
					return False
				print "dbnames = %s" % connection.databaseNames()
				print "tablenames = %s" % connection.tableNames()
				#self.useDatabase(connection, filename)

				self.plugin.connection.kexidbconnection = connection
				self.updateConnectButtons()
				return True

			def disconnectClicked(self):
				if not self.plugin.connection.disconnect():
					qt.QMessageBox.critical(self,"Failed to disconnect",self.plugin.connection.lastError())
					return
				self.updateConnectButtons()

			def updateConnectButtons(self):
				connected = self.plugin.connection.isConnected()
				self.prjbox.setEnabled(not connected)
				self.driverbox.setEnabled(not connected)
				self.connectbtn.setEnabled( (not connected) and (self.driverbox.driver != None) )
				self.disconnectbtn.setEnabled(connected)
				self.tablebox.tablebtn.setEnabled(connected)
				self.fieldbox.tableChanged(self.tablebox.tableedit.text())

			def getOptionValue(self,optionname):
				try:
					if optionname == 'project': return str(self.prjbox.prjcombo.currentText())
					elif optionname == 'driver': return str(self.driverbox.drivercombo.currentText())
					elif optionname == 'file': return str(self.driverbox.fileedit.text())
					elif optionname == 'hostname': return str(self.driverbox.hostedit.text())
					elif optionname == 'port': return str(self.driverbox.portedit.text())
					elif optionname == 'usesocketfile': return str(self.driverbox.sockfilecheckbox.isChecked())
					elif optionname == 'socketfile': return str(self.driverbox.sockfileedit.text())
					elif optionname == 'username': return str(self.driverbox.useredit.text())
					elif optionname == 'password': return str(self.driverbox.passedit.text())
					elif optionname == 'database': return str(self.driverbox.dbedit.text())
					elif optionname == 'table': return str(self.tablebox.tableedit.text())
					elif optionname == 'fields': return str(self.fieldbox.fieldsedit.text())
					elif optionname == 'where': return str(self.whereedit.text())
				except:
					pass
				return ""
				
		mainbox = MainBox(self,plugin,parent)
		plugin.widget = mainbox
		return mainbox




"""
Copy Center

Description:
Python script to copy data between different datastores.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

class CopyCenter:

	class Plugin:
		def __init__(self, plugin):
			self.plugin = plugin
			self.name = plugin.name
			self.source = self.load("Source")
			self.destination = self.load("Destination")

		def load(self, plugintype):
			instance = None
			try:
				if hasattr(self.plugin, plugintype):
					return getattr(self.plugin, plugintype)(self.plugin)
			except:
				import traceback
				print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			return None

	def __init__(self, scriptpath):
		self.scriptpath = scriptpath
		self.homepath = self.getHomePath()
		self.plugins = {}

		import os
		import sys
		if not os.path.exists(scriptpath):
			print "The Path %s does not exist" % scriptpath
		else:
			import re
			regexp = re.compile('^CopyCenterPlugin(.*)\\.py$')
			for f in os.listdir(scriptpath):
				file = os.path.join(scriptpath, f)
				if not os.path.isfile(file): continue
				m = regexp.match(f)
				if not m: continue
				print "Plugin name=%s file=%s" % (m.group(1),file)
				mylocals = {}
				try:
					execfile(file, globals(), mylocals)
					if mylocals.has_key("CopyCenterPlugin"):
						plugin = mylocals.get("CopyCenterPlugin")(self)
						self.plugins[plugin.name] = self.Plugin(plugin)
				except:
					print "Failed to import file=%s" % file
					import traceback
					print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )

	def getHomePath(self):
		""" Return the homedirectory. """
		import os
		try:
			home = os.getenv("HOME")
			if not home:
				import pwd
				user = os.getenv("USER") or os.getenv("LOGNAME")
				if not user:
					pwent = pwd.getpwuid(os.getuid())
				else:
					pwent = pwd.getpwnam(user)
				home = pwent[6]
			return home
		except (KeyError, ImportError):
			return os.curdir

class Copierer:
	def __init__(self): pass
	def appendProgressMessage(self,messagetext): pass
	def writeSuccess(self,record,rowcount): pass
	def writeFailed(self,record): pass

def runGuiApp(copycenter, name):
	import qt
	import sys

	#--------------------------------------------------------------------

	class ListViewDialog(qt.QDialog):
		def __init__(self, parent, caption):
			qt.QDialog.__init__(self, parent, "ProgressDialog", 1)
			self.parent = parent
			self.setCaption(caption)
			layout = qt.QVBoxLayout(self)
			box = qt.QVBox(self)
			box.setMargin(2)
			layout.addWidget(box)
			self.listview = qt.QListView(box)
			self.listview.setAllColumnsShowFocus(True)
			self.listview.header().setStretchEnabled(True,0)
			btnbox = qt.QHBox(box)
			btnbox.setMargin(6)
			btnbox.setSpacing(6)
			self.okbtn = qt.QPushButton(btnbox)
			self.okbtn.setText("Ok")
			#qt.QObject.connect(okbtn, qt.SIGNAL("clicked()"), self.okClicked)
			self.cancelbtn = qt.QPushButton(btnbox)
			self.cancelbtn.setText("Cancel")
			qt.QObject.connect(self.cancelbtn, qt.SIGNAL("clicked()"), self.close)
			box.setMinimumSize(qt.QSize(460,380))
		def addItem(self,valuelist,afteritem = None):
			if afteritem == None:
				item = qt.QListViewItem(self.listview)
			else:
				item = qt.QListViewItem(self.listview,afteritem)
			i = 0
			for value in valuelist:
				item.setText(i,value)
				i += 1
			return item

	#--------------------------------------------------------------------

	class CopyJobWidget(qt.QVBox):
		def __init__(self,dialog,parent):
			self.dialog = dialog
			qt.QVBox.__init__(self,parent)
			self.setSpacing(6)
			typebox = qt.QHBox(self)
			typebox.setSpacing(6)
			label = qt.QLabel("Job File:",typebox)
			self.jobfilecombobox = qt.QComboBox(typebox)
			typebox.setStretchFactor(self.jobfilecombobox,1)
			self.jobfilecombobox.setEditable(True)
			self.jobfilecombobox.insertItem("")
			label.setBuddy(self.jobfilecombobox)
			qt.QObject.connect(self.jobfilecombobox, qt.SIGNAL("textChanged(const QString&)"), self.jobfilecomboboxChanged)

			import os
			import re
			for f in os.listdir(self.dialog.copycenter.homepath):
				file = os.path.join(self.dialog.copycenter.homepath,f)
				if os.path.isfile(file) and re.search(".+\\.copycenterjob.xml$",f):
					self.jobfilecombobox.insertItem(file)

			loadbtn = qt.QPushButton(typebox)
			loadbtn.setText("Open...")
			qt.QObject.connect(loadbtn, qt.SIGNAL("clicked()"), self.openClicked)
			savebtn = qt.QPushButton(typebox)
			savebtn.setText("Save...")
			qt.QObject.connect(savebtn, qt.SIGNAL("clicked()"), self.saveClicked)

			self.listview = qt.QListView(self)
			self.listview.setAllColumnsShowFocus(True)
			self.listview.setSorting(-1)
			self.listview.setDefaultRenameAction(qt.QListView.Reject)
			self.listview.header().setClickEnabled(False)
			self.listview.addColumn("Name")
			self.listview.addColumn("Value")
			qt.QObject.connect(self.listview, qt.SIGNAL("doubleClicked(QListViewItem*, const QPoint&, int)"), self.doubleClicked)
			#qt.QObject.connect(self.listview, qt.SIGNAL("itemRenamed(QListViewItem*, int, const QString&)"), self.itemRenamed)

		def doubleClicked(self, **args):
			print "CopyJobWidget.doubleClicked"
			item = self.listview.selectedItem()
			if item and item.parent(): item.startRename(1)

		def readOptions(self,domnode,plugininst):
			print "CopyJobWidget.readOptions plugintype=\"%s\"" % plugininst.plugintype
			for node in domnode.childNodes:
				if node.nodeType == node.ELEMENT_NODE:
					v = node.getAttribute("value")
					plugininst.options[node.nodeName] = v
					print "Option \"%s\" has value \"%s\" now." % (node.nodeName, v)

		def jobfilecomboboxChanged(self, **args):
			print "CopyJobWidget.jobfilecomboboxChanged"
			import os
			import xml.dom.minidom
			filename = str(self.jobfilecombobox.currentText())
			if not os.path.isfile(filename): return
			domdoc = xml.dom.minidom.parse(filename)
			try:
				elements = domdoc.getElementsByTagName("CopyCenterJob")[0]
				sourcenode = elements.getElementsByTagName("Source")[0]
				destinationnode = elements.getElementsByTagName("Destination")[0]
			except:
				raise "The XML-file \"%s\" does not contain a valid copy-job." % filename
			
			sourcepluginname = str(sourcenode.getAttribute('plugin'))
			if not self.dialog.sourcedata.combobox.listBox().findItem(sourcepluginname,qt.Qt.ExactMatch):
				raise "There exists no plugin with the name \"%s\"." % sourcepluginname
			self.dialog.sourcedata.combobox.setCurrentText(sourcepluginname)

			destinationpluginname = str(destinationnode.getAttribute('plugin'))
			if not self.dialog.destinationdata.combobox.listBox().findItem(destinationpluginname,qt.Qt.ExactMatch):
				raise "There exists no plugin with the name \"%s\"." % destinationpluginname
			self.dialog.destinationdata.combobox.setCurrentText(destinationpluginname)

			self.readOptions(sourcenode,self.dialog.getSourcePluginImpl())
			self.readOptions(destinationnode,self.dialog.getDestinationPluginImpl())
			self.maybeUpdate()

		def openClicked(self):
			text = str(self.jobfilecombobox.currentText())
			if text == "": text = self.dialog.copycenter.homepath
			filename = str(qt.QFileDialog.getOpenFileName(text,"*.copycenterjob.xml;;*",self.dialog))
			if filename != "": self.jobfilecombobox.setCurrentText(filename)

		def escape(self,s):
			return s.replace("&", "&amp;").replace("'", "&apos;").replace("<", "&lt;").replace(">", "&gt;").replace('"', "&quot;")

		def writeOptions(self,writer,pluginname,plugininst):
			print "CopyJobWidget.writeOptions"
			writer.write("<%s plugin=\"%s\">\n" % (plugininst.plugintype, pluginname))
			for optionname in plugininst.options:
				value = self.escape( unicode(plugininst.options[optionname]).encode("utf-8") )
				writer.write("\t<%s value=\"%s\" />\n" % (optionname,value))
			writer.write("</%s>\n" % plugininst.plugintype)

		def saveClicked(self):
			text = str(self.jobfilecombobox.currentText())
			if text == "":
				import os
				text = os.path.join(self.dialog.copycenter.homepath,"default.copycenterjob.xml")
			filename = str(qt.QFileDialog.getSaveFileName(text,"*.copycenterjob.xml;;*",self.dialog))
			if str(filename) == "": return
			f = open(filename, "w")
			f.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
			f.write("<CopyCenterJob>\n")
			sourcepluginname = self.dialog.sourcedata.combobox.currentText()
			self.writeOptions(f, sourcepluginname, self.dialog.getSourcePluginImpl())
			destinationpluginname = self.dialog.destinationdata.combobox.currentText()
			self.writeOptions(f, destinationpluginname, self.dialog.getDestinationPluginImpl())
			f.write("</CopyCenterJob>\n")
			f.close()
			print "File \%s\" successfully written." % filename

		def addItem(self, pluginimpl, afteritem = None, parentitem = None):
			#print "CopyJobWidget.addItem"
			class ListViewItem(qt.QListViewItem):
				def __init__(self, pluginimpl, listview, parentitem = None, afteritem = None):
					self.pluginimpl = pluginimpl
					if parentitem == None:
						qt.QListViewItem.__init__(self,listview)
						self.setOpen(True)
					else:
						if afteritem == None:
							qt.QListViewItem.__init__(self,parentitem)
						else:
							qt.QListViewItem.__init__(self,parentitem,afteritem)
						self.setRenameEnabled(1,True)
				def startRename(self, columnindex):
					qt.QListViewItem.startRename(self,columnindex)
					#lineedit = self.listView().viewport().child("qt_renamebox")
					#if lineedit:
					#	regexp = qt.QRegExp("^[_A-Z]+[_A-Z0-9]*$", False)
					#	v = qt.QRegExpValidator(regexp, self.listView());
					#	lineedit.setValidator(v)
				def okRename(self, columnindex):
					if columnindex == 1:
						n = str(self.text(0))
						if not self.pluginimpl.options.has_key(n):
							raise "No such option \"%s\"" % n
						qt.QListViewItem.okRename(self,columnindex)
						v = str(qt.QListViewItem.text(self,1))
						print "Option \"%s\" has value \"%s\" now." % (n,v)
						self.pluginimpl.options[n] = v

				def text(self, columnindex):
					if columnindex == 1:
						if qt.QListViewItem.text(self,0).contains("password"):
							return "*" * len(str(qt.QListViewItem.text(self,1)))
					return qt.QListViewItem.text(self,columnindex)
			return ListViewItem(pluginimpl, self.listview, parentitem, afteritem)

		def updateItem(self,pluginname,pluginimpl):
			#print "CopyJobWidget.updateItem"
			if pluginimpl == None: return
			#plugin = self.dialog.plugins[pluginname]
			item = self.addItem(pluginimpl)
			item.setText(0,"%s: %s" % (pluginimpl.plugintype, pluginname))
			afteritem = None
			for i in pluginimpl.options:
				afteritem = self.addItem(pluginimpl, afteritem, item)
				afteritem.setText(0,str(i))
				afteritem.setText(1,str(pluginimpl.options[i]))
				print "CopyJobWidget.updateItem Added item with name \"%s\" and value \"%s\"" % (str(i),str(pluginimpl.options[i]))
			pass
		
		def maybeUpdate(self):
			print "CopyJobWidget.maybeUpdate"
			self.listview.clear()
			try:
				self.updateItem(self.dialog.getDestinationPluginName(), self.dialog.getDestinationPluginImpl())
				self.updateItem(self.dialog.getSourcePluginName(), self.dialog.getSourcePluginImpl())
			except:
				import traceback
				print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
				self.listview.clear()

	#--------------------------------------------------------------------

	class ProgressDialog(qt.QDialog):
		def __init__(self, dialog):
			self.dialog = dialog
			self.starttime = None
			qt.QDialog.__init__(self, dialog, "ProgressDialog", 1)
			self.setCaption("Copying...")
			layout = qt.QVBoxLayout(self)
			box = qt.QVBox(self)
			box.setSpacing(6)
			box.setMargin(6)
			layout.addWidget(box)
			self.textbrowser = qt.QTextBrowser(box)
			self.textbrowser.setWordWrap(qt.QTextEdit.WidgetWidth)
			self.textbrowser.setTextFormat(qt.Qt.RichText)
			statusbox = qt.QFrame(box)
			layout = qt.QGridLayout(statusbox,4,2,0,2)
			layout.addWidget(qt.QLabel("Number of records done:",statusbox),0,0)
			self.donecounter = 0
			self.donelabel = qt.QLabel("-",statusbox)
			layout.addWidget(self.donelabel,0,1)
			layout.addWidget(qt.QLabel("Successfully copied records:",statusbox),1,0)
			self.successcounter = 0
			self.successlabel = qt.QLabel("-",statusbox)
			layout.addWidget(self.successlabel,1,1)
			layout.addWidget(qt.QLabel("Failed to copy records:",statusbox),2,0)
			self.failedcounter = 0
			self.failedlabel = qt.QLabel("-",statusbox)
			layout.addWidget(self.failedlabel,2,1)
			layout.addWidget(qt.QLabel("Elapsed time in seconds:",statusbox),3,0)
			self.elapsedlabel = qt.QLabel("-",statusbox)
			layout.addWidget(self.elapsedlabel,3,1)
			btnbox = qt.QHBox(box)
			btnbox.setSpacing(6)
			self.donebtn = qt.QPushButton(btnbox)
			self.donebtn.setText("Done")
			self.donebtn.setEnabled(False)
			qt.QObject.connect(self.donebtn,qt.SIGNAL("clicked()"),self.close)
			self.cancelbtn = qt.QPushButton(btnbox)
			self.cancelbtn.setText("Cancel")
			qt.QObject.connect(self.cancelbtn,qt.SIGNAL("clicked()"),self.close)
			box.setMinimumSize( qt.QSize(500,380) )

		def updateStates(self):
			if self.starttime != None:
				self.donelabel.setText(str(self.donecounter))
				self.failedlabel.setText(str(self.failedcounter))
				self.successlabel.setText(str(self.successcounter))
				self.elapsedlabel.setText( str(self.starttime.elapsed() / 1000) )
				self.donelabel.update()
				self.failedlabel.update()
				self.successlabel.update()
				self.elapsedlabel.update()

		def writeSuccess(self, record, rowcount):
			self.donecounter += rowcount
			self.successcounter += rowcount
			qt.qApp.processEvents()
		def writeFailed(self, record):
			self.donecounter += 1
			self.failedcounter += 1
			qt.qApp.processEvents()

		def startCopy(self):
			try:
				global Copierer
				copierer = Copierer()
				copierer.appendProgressMessage = self.textbrowser.append
				copierer.writeSuccess = self.writeSuccess
				copierer.writeFailed = self.writeFailed

				self.starttime = qt.QTime()
				self.updatetimer = qt.QTimer(self)
				qt.QObject.connect(self.updatetimer,qt.SIGNAL("timeout()"),self.updateStates)

				# Initialize the source
				sourcename = self.dialog.getSourcePluginName()
				sourceimpl = self.dialog.getSourcePluginImpl()
				self.textbrowser.append("Source: %s" % sourcename)
				if sourceimpl == None:
					raise "No such source."
				try:
					sourceimpl.init(copierer)

					# Initialize the destination
					destinationname = self.dialog.getDestinationPluginName()
					destinationimpl = self.dialog.getDestinationPluginImpl()
					self.textbrowser.append("<hr>Destination: %s" % destinationname)
					if destinationimpl == None:
						raise "No such destination."
					try:
						destinationimpl.init(copierer)

						self.starttime.start()
						self.updatetimer.start(500)
						qt.qApp.processEvents()

						# Copy the records
						self.textbrowser.append("<hr><i>Copy the records...</i>")
						while True:
							record = sourceimpl.read()
							if record == None: break
							destinationimpl.write(record)

						self.updateStates()
					finally:
						destinationimpl.finish()
				finally:
					sourceimpl.finish()

				self.setCaption("Copy done")
				self.textbrowser.append("<hr><b>Copy done.</b>")
			except:
				self.setCaption("Copy failed")
				self.textbrowser.append("<b>Error: %s</b>" % sys.exc_info()[0])
				import traceback
				print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			#self.progressbar.setEnabled(False)
			self.donebtn.setEnabled(True)
			self.cancelbtn.setEnabled(False)
			self.updatetimer.stop()
			self.starttime = None

		def show(self):
			qt.QDialog.show(self)
			qt.QTimer.singleShot(10,self.startCopy)
			qt.qApp.processEvents()

		def closeEvent(self, closeevent):
			if not self.dialog.getSourcePluginImpl().isFinished():
				if qt.QMessageBox.warning(self,"Abort?","Abort the copy?",qt.QMessageBox.Yes,qt.QMessageBox.No) != qt.QMessageBox.Yes:
					closeevent.ignore()
					return
				self.dialog.getSourcePluginImpl().finish()
				self.dialog.getDestinationPluginImpl().finish()
			closeevent.accept()

	#--------------------------------------------------------------------

	class DataSelector(qt.QVGroupBox):
		def __init__(self, plugintype, title, caption, parent, dialog, items):
			self.plugintype = plugintype
			self.pluginimpl = None
			self.dialog = dialog
			self.mainbox = None

			qt.QVGroupBox.__init__(self,title,parent)
			self.setInsideMargin(6)
			self.setInsideSpacing(0)

			typebox = qt.QHBox(self)
			label = qt.QLabel(caption,typebox)
			self.combobox = qt.QComboBox(typebox)
			for item in items:
				self.combobox.insertItem(str(item))
			label.setBuddy(self.combobox)
			typebox.setStretchFactor(self.combobox,1)

			self.scrollview = qt.QScrollView(self)
			try:
				self.scrollview.setResizePolicy(qt.QScrollView.AutoOne)
				self.scrollview.setFrameStyle(qt.QFrame.NoFrame);
				self.scrollview.setResizePolicy(qt.QScrollView.AutoOneFit);
				self.scrollview.viewport().setPaletteBackgroundColor(self.paletteBackgroundColor())
			except:
				import traceback
				print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			qt.QObject.connect(self.combobox, qt.SIGNAL("activated(int)"), self.activated)

		def updatePlugin(self):
			print "DataSelector.updatePlugin"
			self.pluginimpl = None
			text = str(self.combobox.currentText())
			plugin = self.dialog.copycenter.plugins[text]
			self.pluginimpl = getattr(plugin, self.plugintype)

		def removeMainBox(self):
			if self.mainbox == None: return
			try:
				self.scrollview.removeChild(self.mainbox)
				self.mainbox.destroy()
			except:
				pass
			self.mainbox = None

		def updateMainBox(self):
			print "DataSelector.updateMainBox"
			self.removeMainBox()
			self.mainbox = qt.QVBox( self.scrollview.viewport() )
			self.mainbox.setSpacing(2)
			if self.pluginimpl != None:
				try:
					self.pluginimpl.createWidget(self.dialog, self.mainbox)
				except:
					import traceback
					print "".join( traceback.format_exception(sys.exc_info()[0],sys.exc_info()[1],sys.exc_info()[2]) )
			self.mainbox.setStretchFactor(qt.QWidget(self.mainbox), 1)
			self.mainbox.show()
			self.scrollview.addChild(self.mainbox)

		def activated(self, **args):
			self.updatePlugin()
			self.updateMainBox()

		def maybeUpdate(self):
			print "DataSelector.maybeUpdate"
			self.removeMainBox()
			qt.QTimer.singleShot(50, self.activated)

		def maybeDone(self):
			print "DataSelector.maybeDone"
			if self.pluginimpl.widget == None: return
			for optionname in self.pluginimpl.options:
				self.pluginimpl.options[optionname] = self.pluginimpl.widget.getOptionValue(optionname)

	#--------------------------------------------------------------------

	class Dialog(qt.QDialog):
		def __init__(self, copycenter, parent):
			self.copycenter = copycenter

			import qt
			import os
			import sys

			self.ListViewDialog = ListViewDialog
			qt.QDialog.__init__(self, parent, "Dialog", 1, qt.Qt.WDestructiveClose)
			self.setCaption("Copy Center")
			layout = qt.QVBoxLayout(self)
			box = qt.QVBox(self)
			box.setMargin(6)
			box.setSpacing(6)
			layout.addWidget(box)
			self.tab = qt.QTabWidget(box)
			self.tab.setMargin(6)
			box.setStretchFactor(self.tab,1)

			self.jobsbox = CopyJobWidget(self,self.tab)
			self.tab.addTab(self.jobsbox,"Jobs")

			self.splitter = qt.QSplitter(self.tab)

			sourceplugins = []
			destinationplugins = []
			for pluginname in self.copycenter.plugins:
				if self.copycenter.plugins[pluginname].source != None:
					sourceplugins.append(pluginname)
				if self.copycenter.plugins[pluginname].destination != None:
					destinationplugins.append(pluginname)
			sourceplugins.sort()
			destinationplugins.sort()

			self.sourcedata = DataSelector(
				"source", # id
				"Read Data From", # title
				"Source:", # caption
				self.splitter, self, sourceplugins)
			self.destinationdata = DataSelector(
				"destination", # id
				"Write Data to", # title
				"Destination:", # caption
				self.splitter, self, destinationplugins)

			btnbox = qt.QHBox(box)
			btnbox.setSpacing(6)
			okbtn = qt.QPushButton(btnbox)
			okbtn.setText("Start Copy")
			okbtn.setDefault(True)
			qt.QObject.connect(okbtn,qt.SIGNAL("clicked()"),self.startCopy)
			cancelbtn = qt.QPushButton(btnbox)
			cancelbtn.setText("Cancel")
			qt.QObject.connect(cancelbtn,qt.SIGNAL("clicked()"),self.close)

			self.tab.addTab(self.splitter,"Copy")
			self.tab.setCurrentPage(1)

			self.helpbrowser = qt.QTextBrowser(self.tab)
			self.helpbrowser.setLinkUnderline(False)
			self.helpbrowser.setUndoRedoEnabled(False)
			self.tab.addTab(self.helpbrowser,"Help")
			qt.QObject.connect(self.tab,qt.SIGNAL("currentChanged(QWidget*)"),self.currentTabChanged)

			box.setMinimumSize( qt.QSize(760,500) )
			
			defaultfile = os.path.join(self.copycenter.homepath,"default.copycenterjob.xml")
			if os.path.isfile(defaultfile):
				print "Reading default copy job file: %s" % defaultfile
				self.jobsbox.jobfilecombobox.setCurrentText(defaultfile)

		def getSourcePluginName(self):
			return str(self.sourcedata.combobox.currentText())
		def getSourcePluginImpl(self):
			return self.copycenter.plugins[self.getSourcePluginName()].source
		def getDestinationPluginName(self):
			return str(self.destinationdata.combobox.currentText())
		def getDestinationPluginImpl(self):
			return self.copycenter.plugins[self.getDestinationPluginName()].destination

		def currentTabChanged(self,widget):
			if self.tab.currentPage() == self.jobsbox:
				# The "Copy" page is done
				self.sourcedata.maybeDone()
				self.destinationdata.maybeDone()
				# Update the "Jobs" page
				self.jobsbox.maybeUpdate()
			elif self.tab.currentPage() == self.splitter:
				# Update the "Copy" page
				self.sourcedata.maybeUpdate()
				self.destinationdata.maybeUpdate()
			elif self.tab.currentPage() == self.helpbrowser and self.helpbrowser.lines() <= 1:
				# Update the "Help" page
				import os
				file = os.path.join(self.copycenter.scriptpath, "readme.html")
				if not os.path.isfile(file): return
				fh = open(file,'r')
				self.helpbrowser.setText( fh.read() )
				fh.close()
				
		def startCopy(self):
			dlg = ProgressDialog(self)
			dlg.show()

	#--------------------------------------------------------------------

	if name == "__main__":
		qtapp = qt.QApplication(sys.argv)
	else:
		qtapp = qt.qApp
	dialog = Dialog(copycenter, qtapp.mainWidget())
	dialog.exec_loop()

import os

if __name__ == "__main__":
	scriptpath = os.getcwd()
else:
	scriptpath = os.path.dirname(__name__)

copycenter = CopyCenter(scriptpath)
runGuiApp(copycenter, __name__)



"""
Import data from a XHTML file to a KexiDB table.

Description:
This script implements import of data from a XHTML file to a KexiDB table. The
table needs to be an already existing table the data should be added to.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

class SaxInput:
	""" The inputsource we like to import the data from. This class
	provides us abstract access to the SAX XML parser we use internaly
	to import data from the XML-file. """

	xmlfile = None
	""" The XML file we should read the content from. """

	def __init__(self):
		""" Constructor. """

		# try to import the xml.sax python module.
		try:
			import xml.sax.saxlib
			import xml.sax.saxexts
		except:
			raise "Import of the python xml.sax.saxlib module failed. This module is needed by the ImportXHTML python script."

	def read(self, outputwriter):
		""" Start reading and parsing the XML-file. """

		import xml.sax.saxlib
		import xml.sax.saxexts

		class SaxHandler(xml.sax.saxlib.HandlerBase):
			""" The SaxHandler is our event-handler SAX calls on
			parsing the XML-file. """

			tablebase = ["html","body","table"]
			""" The table-base defines where we will find our table-tag
			that holds all the data we are interessted at. The default
			is to look at <html><body><table></table></body></html>. """
			
			def __init__(self, inputreader, outputwriter):
				""" Constructor. """

				# The to a SaxInput instance pointing inputreader.
				self.inputreader = inputreader
				# The to a KexiDBOutput instance pointing outputwriter.
				self.outputwriter = outputwriter
				# The hierachy-level in the DOM-tree we are in.
				self.level = 0
				# Defines if we are in the with tablebase defined DOM-element.
				self.intable = False

				# Points to a KexiDBOutput.Record instance if we are in a DOM-element that defines a record.
				self.record = None
				# Points to a KexiDBOutput.Field instance if we are in a record's field.
				self.field = None

			def startDocument(self):
				sys.stdout.write('=> Starting parsing\n')

			def endDocument(self):
				sys.stdout.write('=> Fineshed parsing\n')

			def startElement(self, name, attrs):
				""" This method is called by SAX if a DOM-element starts. """

				if self.level < len(self.tablebase):
					if self.tablebase[self.level] != name:
						self.intable = False
					else:
						self.intable = True
				self.level += 1
				if not self.intable:
					return

				# Print some debugging-output to stdout.
				for idx in range(self.level): sys.stdout.write('  ')
				sys.stdout.write('Element: %s' % name)
				for attrName in attrs.keys():
					sys.stdout.write(' %s="%s"' % (attrName,attrs.get(attrName)))
				sys.stdout.write('\n')

				# handle tr-, th- and td-tags inside the table.
				if name == "tr" and (self.level == len(self.tablebase) + 1):
					self.record = self.outputwriter.Record()
				elif name == "td" and (self.level == len(self.tablebase) + 2):
					self.field = self.outputwriter.Field()
				elif name == "th" and (self.level == len(self.tablebase) + 2):
					self.field = self.outputwriter.Field()

			def endElement(self, name):
				""" This method is called by SAX if a DOM-Element ends. """

				self.level -= 1
				#sys.stdout.write('EndElement:%s level:%s len(self.tablebase):%s\n' % (name,self.level,len(self.tablebase)))

				if self.record != None:
					# a record is defined. so, we are looking for the matching
					# end-tags to close a record or a field.
					if name == "tr" and (self.level == len(self.tablebase)):
						self.outputwriter.write(self.record)
						self.record = None
						self.field = None
					elif name == "td" and (self.level == len(self.tablebase) + 1):
						#if self.field == None:
						#	raise "Unexpected closing </td>"
						self.record.setField( self.field )
						self.field = None
					elif name == "th" and (self.level == len(self.tablebase) + 1):
						#if self.field == None:
						#	raise "Unexpected closing </td>"
						self.record.setHeader( self.field )
						self.field = None

			def characters(self, chars, offset, length):
				""" This method is called by SAX if the text-content of a DOM-Element
				was parsed. """

				if self.field != None:
					# the xml-data is unicode and we need to encode it
					# to latin-1 cause KexiDB deals only with latin-1.
					u = unicode(chars[offset:offset+length])
					self.field.append(u.encode("latin-1"))

		# start the job
		outputwriter.begin()
		# create saxhandler to handle parsing events.
		handler = SaxHandler(self, outputwriter)
		# we need a sax-parser and connect it with the handler.
		parser = xml.sax.saxexts.make_parser()
		parser.setDocumentHandler(handler)
		# open the XML-file, parse the content and close the file again.
		f = file(self.xmlfile, 'r')
		parser.parseFile(f)
		f.close()
		# job is done
		outputwriter.end()

class KexiDBOutput:
	""" The destination target we like to import the data to. This class
	provides abstract access to the KexiDB module. """

	class Result:
		""" Holds some informations about the import-result. """
		def __init__(self, outputwriter):
			self.outputwriter = outputwriter
			# number of records successfully imported.
			self.successcount = 0
			# number of records where import failed.
			self.failedcount = 0
			
		def addLog(self, record, state):
			import datetime
			date = datetime.datetime.now().strftime("%Y-%m-%d %H:%M.%S")
			self.outputwriter.logfile.write("%s (%s) %s\n" % (date,state,str(record)))

		def success(self, record):
			""" Called if a record was written successfully. """
			print "SUCCESS: %s" % str(record)
			self.successcount += 1
			if hasattr(self.outputwriter,"logfile"):
				self.addLog(record, "Success")

		def failed(self, record):
			""" Called if we failed to write a record. """
			print "FAILED: %s" % str(record)
			self.failedcount += 1
			if hasattr(self.outputwriter,"logfile"):
				self.addLog(record, "Failed")

	class Record:
		""" A Record in the dataset. """
		def __init__(self):
			self.fields = []
		def setHeader(self, headerfield):
			self.fields.append( headerfield )
			self.isHeader = True
		def setField(self, field):
			self.fields.append( field )
		def __str__(self):
			s = "["
			for f in self.fields:
				s += "%s, " % str(f)
			return s + "]"

	class Field:
		""" A field in a record. """
		def __init__(self):
			self.content = []
		def append(self, content):
			self.content.append( content )
		def __str__(self):
			return "".join(self.content)

	def __init__(self):
		""" Constructor. """
		import kexiapp
		keximainwindow = kexiapp.get("KexiAppMainWindow")
		
                try:
			self.connection = keximainwindow.getConnection()
		except:
			raise "No connection established. Please open a project before."

		self.fieldlist = None
		self.headerrecord = None
		self.mapping = {}

	def begin(self):
		""" Called before parsing starts. """
		print "START JOB"
		if self.fieldlist == None:
			raise "Invalid tableschema or fieldlist!"
		global KexiDBOutput
		self.result = KexiDBOutput.Result(self)
		if hasattr(self,"logfilename") and self.logfilename != None and self.logfilename != "":
			self.logfile = open(self.logfilename,'w')

	def end(self):
		""" Called if parsing is fineshed. """
		print "END JOB"
		self.logfile = None
		self.mapping = {}
		#self.headerrecord = None

	def getTables(self):
		""" return a list of avaiable tablenames. """
		tables = self.connection.tableNames()
		tables.sort()
		return tables

	def setTable(self, tablename):
		""" Set the tablename we like to import the data to. """
		tableschema = self.connection.tableSchema(tablename)
		if tableschema == None:
			raise "There exists no table with the name '%s'!" % tablename
		self.fieldlist = tableschema.fieldlist()
		fields = self.fieldlist.fields()
		for field in fields:
			print "KexiDBOutput.setTable(%s): %s(%s)" % (tablename,field.name(),field.type())
		print "names=%s" % self.fieldlist.names()
		
	def setMapping(self, mapping):
		""" Set the tablefieldname=xmlcolnr dictonary we should map the data to. """
		self.mapping = mapping

	def setLogFile(self, logfilename):
		""" Set the name of the logfile. """
		self.logfilename = logfilename

	def write(self, record):
		""" Write the record to the KexiDB table. """

		if hasattr(record, "isHeader"):
			self.headerrecord = record
			return

		sys.stdout.write('KexiDBOutput.write:')
		for f in record.fields:
			sys.stdout.write(' "%s"' % f)
		sys.stdout.write('\n')

		if hasattr(self,"onWrite"):
			if not self.onWrite(record):
				raise RuntimeError()
			delattr(self,"onWrite")
			self.fieldlist = self.fieldlist.subList( list( self.mapping ) )

		# Translate a KexiDBOutput.Record into a list of values.
		values = []
		for k in self.fieldlist.names():
			values.append( str(record.fields[ int(self.mapping[k]) ]) )
		print "Import values: %s" % values

		try:
			if self.connection.insertRecord(self.fieldlist, values):
				self.result.success(record)
			else:
				self.result.failed(record)
		except:
			err = self.connection.lastError()
			raise Exception( "Failed to insert the record:\n%s\n\n%s" % (values,err) )
			#raise Exception( "Failed to insert into table \"%s\" the record:\n%s\n%s" % (self.tableschema.name(),values,self.connection.lastError()) )

class GuiApp:
	""" The GUI-dialog displayed to let the user define the source
	XML-file and the destination KexiDB table. """

	class InitialDialog:
		def __init__(self, guiapp):
			self.guiapp = guiapp
			self.ok = False

			import gui
			self.dialog = gui.Dialog("Import XHTML")
			self.dialog.addLabel(self.dialog, "Import data from a XHTML-file to a KexiDB table.\n"
                                                  "The destination table needs to be an existing table the data should be added to.")
			self.importfile = self.dialog.addFileChooser(self.dialog,
				"Source File:",
				gui.getHome() + "/kexidata.xhtml",
				(('XHTML files', '*.xhtml'),('All files', '*')))

			self.desttable = self.dialog.addList(self.dialog, "Destination Table:", self.guiapp.outputwriter.getTables())

			#self.operation = self.dialog.addList(self.dialog, "Operation:", ("Insert","Update","Insert/Update"))
			#self.error = self.dialog.addList(self.dialog, "On error:", ("Ask","Skip","Abort"))

			self.logfile = self.dialog.addFileChooser(self.dialog,
				"Log File:",
				"",
				(('Logfiles', '*.log'),('All files', '*')))

			btnframe = self.dialog.addFrame(self.dialog)
			self.dialog.addButton(btnframe, "Next", self.doNext)
			self.dialog.addButton(btnframe, "Cancel", self.doCancel)
			self.dialog.show()

		def doCancel(self):
			""" Called if the Cancel-button was pressed. """
			self.dialog.close()
			self.dialog = None
			#self.guiapp.InitialDialog
			
		def doNext(self):
			""" Start to import the XML-file into the KexiDB table. """

			self.guiapp.inputreader.xmlfile = str(self.importfile.get())
			self.guiapp.outputwriter.setTable( str(self.desttable.get()) )
			self.guiapp.outputwriter.setLogFile( str(self.logfile.get()) )

			try:
				self.guiapp.inputreader.read( self.guiapp.outputwriter )
				
				msgbox = self.dialog.showMessageBox("info","Import done",
					"Successfully imported records: %s\nFailed to import records: %s" % (self.guiapp.outputwriter.result.successcount, self.guiapp.outputwriter.result.failedcount) )
				msgbox.show()
			
				self.doCancel()
			except RuntimeError, e:
				pass
			#except Exception, e:
			#	import traceback
			#	traceback.print_exc()
			#	msgbox = self.dialog.showMessageBox("error", "Error", e)
			#	msgbox.show()

	class MapperDialog:
		""" The dialog that provides us a way to map
		XHTML columns to the destination table. """

		def __init__(self, outputwriter, record):
			self.outputwriter = outputwriter
			self.ok = False
			fieldlist = outputwriter.fieldlist

			import gui
			self.dlg = gui.Dialog("Import XHTML")
			self.dlg.addLabel(self.dlg, "Define how the destination table should be mapped to the data from the XHTML file.")
			values = ["",]
			for i in range(len(record.fields)):
				try:
					values.append( "%s: %s" % (i,str(outputwriter.headerrecord.fields[i])) )
				except:
					values.append( "%s: (%s)" % (i,str(record.fields[i])) )

			self.items = []
			i = 0
			for field in fieldlist.fields():
				f = self.dlg.addFrame(self.dlg)

				l = self.dlg.addList(f, "%s:" % field.name(), values)
				self.items.append( (field,l) )

				details = "%s:" % str( field.type() )
				if field.isAutoInc(): details += "autoinc,"
				if field.isUniqueKey(): details += "unique,"
				if field.isNotNull(): details += "notnull,"
				if field.isNotEmpty(): details += "notempty,"
				self.dlg.addLabel(f, "(%s)" % details[:-1])

				try:
					variable = str( record.fields[i] )
					try:
						int(variable)
						i += 1
						if not field.isAutoInc():
							l.set(i)
					except ValueError, e:
						if not field.type() in ("Integer","BigInteger","ShortInteger","Float","Double"):
							i += 1
							l.set(i)
				except:
					pass

			btnframe = self.dlg.addFrame(self.dlg)
			self.dlg.addButton(btnframe, "Next", self.doNext)
			self.dlg.addButton(btnframe, "Cancel", self.dlg.close)
			self.dlg.show()

		def doNext(self):
			mapping = {}
			for item in self.items:
				(field,l) = item
				fieldname = field.name()
				colnr = str( l.get() ).split(":",1)[0]
				if colnr.isdigit():
					print "Table field '%s' is mapped to XML column '%s'" % (fieldname,colnr)
					mapping[ fieldname ] = colnr
			self.outputwriter.setMapping(mapping)
			self.ok = True
			self.dlg.close()

	def __init__(self, inputreader, outputwriter):
		""" Constructor. """

		self.inputreader = inputreader
		self.outputwriter = outputwriter
		self.outputwriter.onWrite = self.onWrite

		self.InitialDialog(self)

	def onWrite(self, record):
		""" This method got called after the first record got
		readed and before we start to import. """
		return self.MapperDialog(self.outputwriter, record).ok

GuiApp( SaxInput(), KexiDBOutput() )



"""
Initializer for the krosskexiapp-module.

Description:
This module provides the entry-point for python scripts
to work with a running Kexi application instance.

Author:
Sebastian Sauer <mail@dipe.org>

Copyright:
Dual-licensed under LGPL v2+higher and the BSD license.
"""

try:
	import krosskexiapp
except ImportError, e:
	raise "Import of the Kross KexiApp module failed.\n%s" % e

def get(modulename):
	return krosskexiapp.get(modulename)

def currentConnection():
	mainwindow = krosskexiapp.get("KexiAppMainWindow")
	return mainwindow.getConnection()
